from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from App.Common import package_home
from DateTime import DateTime
from UserDict import UserDict
import ExtensionClass
import os

PACKAGE_HOME = package_home(globals())


class FakeRequestSession(ExtensionClass.Base, UserDict):
    """Dummy dict like object with set method for SESSION and REQUEST
    """
    security = ClassSecurityInfo()
    security.setDefaultAccess('allow')
    security.declareObjectPublic()

    def __init__(self):
        UserDict.__init__(self)
        # add a dummy because request mustn't be empty for test
        # like 'if REQUEST:'
        self['__dummy__'] = None

    def __nonzero__(self):
        return True

    def set(self, key, value):
        self[key] = value

InitializeClass(FakeRequestSession)
FakeRequestSession()


def dcEdit(obj):
    """dublin core edit (inplace)
    """
    obj.setTitle('Test title')
    obj.setDescription('Test description')
    obj.setSubject('Test subject')
    obj.setContributors(('test user a',))
    obj.setEffectiveDate(DateTime() - 1)
    obj.setExpirationDate(DateTime() + 2)
    obj.setFormat('text/structured')
    obj.setLanguage('de')
    obj.setRights('GPL')

from Products.validation import ValidationChain
EmptyValidator = ValidationChain('isEmpty')
EmptyValidator.appendSufficient('isEmpty')
idValidator = ValidationChain('isValidId')
idValidator.appendSufficient('isEmptyNoError')
idValidator.appendRequired('isValidId')
TidyHTMLValidator = ValidationChain('isTidyHtmlChain')
TidyHTMLValidator.appendRequired('isTidyHtmlWithCleanup')
NotRequiredTidyHTMLValidator = ValidationChain('isTidyHtmlNotRequiredChain')
NotRequiredTidyHTMLValidator.appendSufficient('isEmptyNoError')
NotRequiredTidyHTMLValidator.appendRequired('isTidyHtmlWithCleanup')
URLValidator = ValidationChain('isURL')
URLValidator.appendSufficient('isEmptyNoError')
URLValidator.appendRequired('isURL')
EmailValidator = ValidationChain('isEmailChain')
EmailValidator.appendSufficient('isEmptyNoError')
EmailValidator.appendSufficient('isMailto')
EmailValidator.appendRequired('isEmail')
EmailValidator = ValidationChain('isEmailChain')
EmailValidator.appendSufficient('isEmptyNoError')
EmailValidator.appendRequired('isEmail')
PhoneValidator = ValidationChain('isPhoneChain')
PhoneValidator.appendSufficient('isEmptyNoError')
PhoneValidator.appendRequired('isInternationalPhoneNumber')

PREFIX = os.path.abspath(os.path.dirname(__file__))


def input_file_path(file):
    return os.path.join(PREFIX, 'input', file)



import urlparse
from urllib import quote

from zope.interface import implements

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import StringWidget

from Products.ATContentTypes.config import PROJECTNAME
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.interfaces import IATLink

from Products.ATContentTypes import ATCTMessageFactory as _

ATLinkSchema = ATContentTypeSchema.copy() + Schema((
    StringField('remoteUrl',
        required=True,
        searchable=True,
        primary=True,
        default="http://",
        # either mailto, absolute url or relative url
        validators=(),
        widget=StringWidget(
            description='',
            label=_(u'label_url', default=u'URL'),
            maxlength='511',
            )),
    ))
finalizeATCTSchema(ATLinkSchema)


class ATLink(ATCTContent):
    """A link to an internal or external resource."""

    schema = ATLinkSchema

    portal_type = 'Link'
    archetype_name = 'Link'
    _atct_newTypeFor = {'portal_type': 'CMF Link', 'meta_type': 'Link'}
    assocMimetypes = ()
    assocFileExt = ('link', 'url', )
    cmf_edit_kws = ('remote_url', )

    implements(IATLink)

    security = ClassSecurityInfo()

    security.declareProtected(ModifyPortalContent, 'setRemoteUrl')
    def setRemoteUrl(self, value, **kwargs):
        """remute url mutator

        Use urlparse to sanify the url
        Also see http://dev.plone.org/plone/ticket/3296
        """
        if value:
            value = urlparse.urlunparse(urlparse.urlparse(value))
        self.getField('remoteUrl').set(self, value, **kwargs)

    security.declareProtected(View, 'remote_url')
    def remote_url(self):
        """CMF compatibility method
        """
        return self.getRemoteUrl()

    security.declarePrivate('cmf_edit')
    def cmf_edit(self, remote_url=None, **kwargs):
        if not remote_url:
            remote_url = kwargs.get('remote_url', None)
        self.update(remoteUrl=remote_url, **kwargs)

    security.declareProtected(View, 'getRemoteUrl')
    def getRemoteUrl(self):
        """Sanitize output
        """
        value = self.Schema()['remoteUrl'].get(self)
        if not value: value = ''  # ensure we have a string
        return quote(value, safe='?$#@/:=+;$,&%')

registerATCT(ATLink, PROJECTNAME)



from zope.interface import implements

from Missing import MV

from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import BooleanWidget

from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import FIELD_INDICES
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics

from Products.ATContentTypes import ATCTMessageFactory as _

ATBooleanCriterionSchema = ATBaseCriterionSchema + Schema((
    BooleanField('bool',
                required=1,
                mode="rw",
                write_permission=ChangeTopics,
                default=None,
                widget=BooleanWidget(
                    label=_(u'label_boolean_criteria_bool', default=u'Value'),
                    description=_(u'help_boolean_criteria_bool',
                                  default=u'True or false')
                    ),
                ),
    ))


class ATBooleanCriterion(ATBaseCriterion):
    """A boolean criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATBooleanCriterionSchema
    meta_type = 'ATBooleanCriterion'
    archetype_name = 'Boolean Criterion'
    shortDesc = 'Boolean (True/False)'

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        result = []
        if self.getBool():
            value = [1, True, '1', 'True']
        else:
            value = [0, '', False, '0', 'False', None, (), [], {}, MV]
        result.append((self.Field(), value))

        return tuple(result)

registerCriterion(ATBooleanCriterion, FIELD_INDICES + ('BooleanIndex', ))



import os

from ZConfig.datatypes import Registry
from ZConfig.loader import SchemaLoader
from Products.ATContentTypes.configuration import datatype

# schema file
DIR = os.path.dirname(os.path.abspath(__file__))
SCHEMA_FILE_NAME = 'schema.xml'
SCHEMA_FILE = os.path.join(DIR, SCHEMA_FILE_NAME)

# registry
# ATCT is using its own datatypes registry to add additional
# handlers.
atctRegistry = Registry()
atctRegistry.register('permission', datatype.permission_handler)
atctRegistry.register('identifer_none', datatype.identifier_none)
atctRegistry.register('byte-size-in-mb', datatype.byte_size_in_mb)
atctRegistry.register('image-dimension', datatype.image_dimension)
atctRegistry.register('image-dimension-or-no', datatype.image_dimension_or_no)
atctRegistry.register('pil-algo', datatype.pil_algo)

# schema
atctSchema = None


def loadSchema(file, registry=atctRegistry, overwrite=False):
    """Loads a schema file

    * file
      A path to a file
    * registry
      A ZConfig datatypes registry instance
    * overwrite
      Overwriting the existing global schema is not possible unless overwrite
      is set to true. Useful only for unit testing.
    """
    global atctSchema
    if atctSchema is not None and not overwrite:
        raise RuntimeError, 'Schema is already loaded'
    schemaLoader = SchemaLoader(registry=registry)
    atctSchema = schemaLoader.loadURL(file)
    return atctSchema

loadSchema(SCHEMA_FILE)

__all__ = ('atctSchema',)



from Products.ATContentTypes.interfaces import IATCTTool
from Products.GenericSetup.utils import PropertyManagerHelpers
from Products.GenericSetup.utils import XMLAdapterBase
from Products.GenericSetup.utils import exportObjects
from Products.GenericSetup.utils import importObjects
from Products.CMFCore.utils import getToolByName


def safeGetAttribute(node, attribute):
    """Get an attribute froma node, but return None if it does not exist.
    """
    if not node.hasAttribute(attribute):
        return None
    else:
        return node.getAttribute(attribute)


class ATCTToolXMLAdapter(XMLAdapterBase, PropertyManagerHelpers):
    """Node in- and exporter for ATCTTool.
    """
    __used_for__ = IATCTTool

    def _exportNode(self):
        """Export the object as a DOM node.
        """
        node = self._doc.createElement('atcttool')
        node.appendChild(self._extractSettings())
        node.appendChild(self._extractProperties())

        self._logger.info('ATCTTool settings exported.')
        return node

    def _importNode(self, node):
        purge = self.environ.shouldPurge()
        if node.getAttribute('purge'):
            purge = self._convertToBoolean(node.getAttribute('purge'))

        if purge:
            self._purgeSettings()
            self._purgeProperties()

        self._initSettings(node)
        self._initProperties(node)
        self._logger.info('ATCTTool settings imported.')

    def _purgeSettings(self):
        # initialize topic tool
        self.context.topic_indexes = {}
        self.context.topic_metadata = {}
        self.context.allowed_portal_types = []
        self.context.createInitialIndexes()
        self.context.createInitialMetadata()

    def _initSettings(self, node):
        for child in node.childNodes:
            if child.nodeName == 'topic_indexes':
                for indexNode in child.childNodes:
                    if indexNode.nodeName == 'index':
                        name = indexNode.getAttribute('name')
                        if indexNode.hasAttribute("remove"):
                            self.context.removeIndex(name)
                            continue

                        try:
                            self.context.getIndex(name)
                        except AttributeError:
                            self.context.addIndex(name)

                        description = safeGetAttribute(indexNode, 'description')
                        enabled = safeGetAttribute(indexNode, 'enabled')
                        if enabled is not None:
                            enabled = self._convertToBoolean(enabled)
                        friendlyName = safeGetAttribute(indexNode, 'friendlyName')

                        criteria = None
                        for critNode in indexNode.childNodes:
                            if critNode.nodeName == 'criteria':
                                for textNode in critNode.childNodes:
                                    if textNode.nodeName != '#text' or \
                                        not textNode.nodeValue.strip():
                                        continue
                                    if criteria is None:
                                        criteria = []
                                    criteria.append(str(textNode.nodeValue))

                        self.context.updateIndex(name,
                                              friendlyName=friendlyName,
                                              description=description,
                                              enabled=enabled,
                                              criteria=criteria)

            if child.nodeName == 'topic_metadata':
                for metadataNode in child.childNodes:
                    if metadataNode.nodeName == 'metadata':
                        name = metadataNode.getAttribute('name')
                        if metadataNode.hasAttribute("remove"):
                            self.context.removeMetadata(name)
                            continue

                        try:
                            self.context.getMetadata(name)
                        except AttributeError:
                            self.context.addMetadata(name)

                        description = safeGetAttribute(metadataNode, 'description')
                        enabled = safeGetAttribute(metadataNode, 'enabled')
                        if enabled is not None:
                            enabled = self._convertToBoolean(enabled)
                        friendlyName = safeGetAttribute(metadataNode, 'friendlyName')
                        self.context.updateMetadata(name,
                                                 friendlyName=friendlyName,
                                                 description=description,
                                                 enabled=enabled)

    def _extractSettings(self):
        fragment = self._doc.createDocumentFragment()
        # topic tool indexes
        indexes = self._doc.createElement('topic_indexes')
        for indexname in self.context.getIndexes():
            index = self.context.getIndex(indexname)
            child = self._doc.createElement('index')
            child.setAttribute('name', str(indexname))
            child.setAttribute('friendlyName', str(index.friendlyName))
            child.setAttribute('description', str(index.description))
            child.setAttribute('enabled', str(bool(index.enabled)))
            for criteria in index.criteria:
                if criteria != 'criterion':
                    sub = self._doc.createElement('criteria')
                    sub.appendChild(self._doc.createTextNode(criteria))
                    child.appendChild(sub)
            indexes.appendChild(child)
        fragment.appendChild(indexes)
        # topic tool metadata
        metadata = self._doc.createElement('topic_metadata')
        for metaname in self.context.getAllMetadata():
            meta = self.context.getMetadata(metaname)
            child = self._doc.createElement('metadata')
            child.setAttribute('name', str(metaname))
            child.setAttribute('friendlyName', str(meta.friendlyName))
            child.setAttribute('description', str(meta.description))
            child.setAttribute('enabled', str(bool(meta.enabled)))
            metadata.appendChild(child)
        fragment.appendChild(metadata)

        return fragment


def importATCTTool(context):
    """Import ATCT Tool configuration.
    """
    site = context.getSite()
    tool = getToolByName(site, 'portal_atct', None)

    if tool is not None:
        importObjects(tool, '', context)


def exportATCTTool(context):
    """Export ATCT Tool configuration.
    """
    site = context.getSite()
    tool = getToolByName(site, 'portal_atct', None)
    if tool is None:
        logger = context.getLogger("atcttool")
        logger.info("Nothing to export.")
        return

    exportObjects(tool, '', context)



import os.path
import sys
import logging

logger = logging.getLogger('ATCT')

ATCT_DIR = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(3, os.path.join(ATCT_DIR, 'thirdparty'))

from AccessControl import ModuleSecurityInfo

from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from Products.ATContentTypes.config import SKINS_DIR
from Products.ATContentTypes.config import PROJECTNAME
from Products.ATContentTypes.config import GLOBALS

from Products.CMFCore.utils import ContentInit
from Products.CMFCore.utils import ToolInit
from Products.CMFCore.DirectoryView import registerDirectory

# Import "ATCTMessageFactory as _" to create messages in atcontenttypes domain
from zope.i18nmessageid import MessageFactory
ATCTMessageFactory = MessageFactory('atcontenttypes')
ModuleSecurityInfo('Products.ATContentTypes').declarePublic('ATCTMessageFactory')

# first level imports: configuration and validation
import Products.ATContentTypes.configuration
import Products.ATContentTypes.lib.validators

# second leven imports: content types, criteria
# the content types are depending on the validators and configuration
import Products.ATContentTypes.content
import Products.ATContentTypes.criteria

# misc imports
from Products.ATContentTypes.tool.atct import ATCTTool

# wire the add permission after all types are registered
from Products.ATContentTypes.permission import wireAddPermissions
wireAddPermissions()

registerDirectory(SKINS_DIR, GLOBALS)


def initialize(context):
    # process our custom types
    if HAS_LINGUA_PLONE:
        from Products.LinguaPlone.public import process_types
        from Products.LinguaPlone.public import listTypes
    else:
        from Products.Archetypes.atapi import process_types
        from Products.Archetypes.atapi import listTypes

    ToolInit(
        'ATContentTypes tool',
        tools=(ATCTTool,),
        icon='tool.gif', ).initialize(context)

    listOfTypes = listTypes(PROJECTNAME)

    content_types, constructors, ftis = process_types(
        listOfTypes,
        PROJECTNAME)

    # Assign an own permission to all content types
    # Heavily based on Bricolite's code from Ben Saller
    from Products.ATContentTypes.permission import permissions

    allTypes = zip(content_types, constructors)
    for atype, constructor in allTypes:
        kind = "%s: %s" % (PROJECTNAME, atype.archetype_name)
        ContentInit(
            kind,
            content_types=(atype,),
            permission=permissions[atype.portal_type],
            extra_constructors=(constructor,),
            ).initialize(context)



from zope.interface import implements

from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import BooleanWidget
from Products.Archetypes.atapi import ReferenceField
from Products.Archetypes.Referenceable import Referenceable

from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import PATH_INDICES
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics

from archetypes.referencebrowserwidget import ReferenceBrowserWidget

from Products.ATContentTypes import ATCTMessageFactory as _

ATPathCriterionSchema = ATBaseCriterionSchema + Schema((
    ReferenceField('value',
                required=1,
                mode="rw",
                write_permission=ChangeTopics,
                accessor="Value",
                mutator="setValue",
                allowed_types_method="getNavTypes",
                multiValued=True,
                keepReferencesOnCopy=True,
                relationship="paths",
                widget=ReferenceBrowserWidget(
                    allow_search=1,
                    label=_(u'label_path_criteria_value', default=u'Folders'),
                    description=_(u'help_path_criteria_value',
                                  default=u'Folders to search in.'),
                    base_query={'is_folderish': True},
                    restrict_browse=True,
                    startup_directory='../')
                ),
    BooleanField('recurse',
                mode="rw",
                write_permission=ChangeTopics,
                accessor="Recurse",
                default=False,
                widget=BooleanWidget(
                    label=_(u'label_path_criteria_recurse', default=u'Search Sub-Folders'),
                    description='',
                    ),
                ),
    ))


class ATPathCriterion(ATBaseCriterion):
    """A path criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATPathCriterionSchema
    meta_type = 'ATPathCriterion'
    archetype_name = 'Path Criterion'
    shortDesc = 'Location in site'

    def getNavTypes(self):
        ptool = self.plone_utils
        nav_types = ptool.typesToList()
        return nav_types

    # Override reference mutator, so that it always reindexes
    def setValue(self, value):
        self.getField('value').set(self, value)
        self.reindexObject()

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        result = []
        depth = (not self.Recurse() and 1) or -1
        paths = ['/'.join(o.getPhysicalPath()) for o in self.Value() if o is not None]

        if paths is not '':
            result.append((self.Field(), {'query': paths, 'depth': depth}))

        return tuple(result)

    # We need references, so we need to be partly cataloged
    _catalogUID = Referenceable._catalogUID
    _catalogRefs = Referenceable._catalogRefs
    _unregister = Referenceable._unregister
    _updateCatalog = Referenceable._updateCatalog
    _referenceApply = Referenceable._referenceApply
    _uncatalogUID = Referenceable._uncatalogUID
    _uncatalogRefs = Referenceable._uncatalogRefs

    def reindexObject(self, *args, **kwargs):
        self._catalogUID(self)
        self._catalogRefs(self)

    def unindexObject(self, *args, **kwargs):
        self._uncatalogUID(self)
        self._uncatalogRefs(self)

    def indexObject(self, *args, **kwargs):
        self._catalogUID(self)
        self._catalogRefs(self)

registerCriterion(ATPathCriterion, PATH_INDICES)



from Products.ATContentTypes.tests import atcttestcase

from Products.CMFCore.utils import getToolByName
from Products.PloneTestCase.setup import default_user
from Products.PloneTestCase.setup import default_password
from Products.PloneTestCase.setup import portal_owner

from Products.ATContentTypes.config import HAS_LINGUA_PLONE


class IntegrationTestCase(atcttestcase.ATCTFunctionalSiteTestCase):

    views = ()

    def afterSetUp(self):
        super(IntegrationTestCase, self).afterSetUp()

        # basic data
        self.folder_url = self.folder.absolute_url()
        self.folder_path = '/%s' % self.folder.absolute_url(1)
        self.basic_auth = '%s:%s' % (default_user, default_password)
        self.owner_auth = '%s:%s' % (portal_owner, default_password)

        # disable portal_factory as it's a nuisance here
        self.portal.portal_factory.manage_setPortalFactoryTypes(listOfTypeIds=[])

        # object
        self.setupTestObject()

    def setupTestObject(self):
        self.obj_id = 'test_object'
        self.obj = None
        self.obj_url = self.obj.absolute_url()
        self.obj_path = '/%s' % self.obj.absolute_url(1)


class ATCTIntegrationTestCase(IntegrationTestCase):
    """Integration tests for view and edit templates
    """

    portal_type = None

    def setupTestObject(self):
        # create test object
        self.obj_id = 'test_object'
        self.title = u'test \xf6bject'
        self.folder.invokeFactory(self.portal_type, self.obj_id, title=self.title)
        self.obj = getattr(self.folder.aq_explicit, self.obj_id)
        self.obj_url = self.obj.absolute_url()
        self.obj_path = '/%s' % self.obj.absolute_url(1)

    def test_createObject(self):
        # create an object using the createObject script
        response = self.publish(self.folder_path +
                                '/createObject?type_name=%s' % self.portal_type,
                                self.basic_auth)

        self.assertEqual(response.getStatus(), 302)  # Redirect to edit

        body = response.getBody()

        self.assertTrue(body.startswith(self.folder_url), body)
        # The url may end with /edit or /atct_edit depending on method aliases
        self.assertTrue(body.endswith('edit'), body)

        # Perform the redirect
        edit_form_path = body[len(self.app.REQUEST.SERVER_URL):]
        response = self.publish(edit_form_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK
        temp_id = body.split('/')[-2]

        new_obj = getattr(self.folder.portal_factory, temp_id)
        self.assertEqual(self.obj.checkCreationFlag(), True)  # object is not yet edited

    def check_newly_created(self):
        """Objects created programmatically should not have the creation flag set"""
        self.assertEqual(self.obj.checkCreationFlag(), False)  # object is fully created

    def test_edit_view(self):
        # edit should work
        response = self.publish('%s/atct_edit' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    def test_base_view(self):
        # base view should work
        response = self.publish('%s/base_view' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    def test_dynamic_view(self):
        # dynamic view magic should work
        response = self.publish('%s/view' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    def test_local_sharing_view(self):
        # sharing tab should work
        response = self.publish('%s/sharing' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    # LinguaPlone specific tests
    if HAS_LINGUA_PLONE:

        def test_linguaplone_views(self):
            response = self.publish('%s/translate_item' % self.obj_path, self.basic_auth)
            self.assertEqual(response.getStatus(), 200)  # OK
            response = self.publish('%s/manage_translations_form' % self.obj_path, self.basic_auth)
            self.assertEqual(response.getStatus(), 200)  # OK

        def test_linguaplone_create_translation(self):
            # create translation creates a new object
            response = self.publish('%s/createTranslation?language=de&set_language=de'
                                     % self.obj_path, self.basic_auth)
            self.assertEqual(response.getStatus(), 302)  # Redirect

            body = response.getBody()
            self.assertTrue(body.startswith(self.folder_url))
            self.assertTrue(body.endswith('/translate_item'))

            # Perform the redirect
            form_path = body[len(self.app.REQUEST.SERVER_URL):]
            response = self.publish(form_path, self.basic_auth)
            self.assertEqual(response.getStatus(), 200)  # OK

            translated_id = "%s-de" % self.obj_id
            self.assertTrue(translated_id in self.folder,
                            self.folder)

    def test_additional_view(self):
        # additional views:
        for view in self.views:
            response = self.publish('%s/%s' % (self.obj_path, view), self.basic_auth)
            self.assertEqual(response.getStatus(), 200,
                "%s: %s" % (view, response.getStatus()))  # OK

    def test_discussion(self):
        # enable discussion for the type
        ttool = getToolByName(self.portal, 'portal_types')
        ttool[self.portal_type].allow_discussion = True

        response = self.publish('%s/discussion_reply_form'
                                 % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # ok

        response = self.publish('%s/discussion_reply?subject=test&body_text=testbody'
                                 % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 302)  # Redirect

        body = response.getBody()
        self.assertTrue(body.startswith(self.folder_url))

        self.assertTrue(hasattr(self.obj.aq_explicit, 'talkback'))

        form_path = body[len(self.app.REQUEST.SERVER_URL):]
        discussionId = form_path.split('#')[1]

        reply = self.obj.talkback.getReply(discussionId)
        self.assertEqual(reply.title, 'test')
        self.assertEqual(reply.text, 'testbody')

    def test_dynamicViewContext(self):
        # register and add a testing template (it's a script)
        self.setRoles(['Manager', 'Member'])

        ttool = self.portal.portal_types
        fti = getattr(ttool, self.portal_type)
        view_methods = fti.getAvailableViewMethods(self.obj) + ('unittestGetTitleOf',)
        fti.manage_changeProperties(view_methods=view_methods)

        self.obj.setLayout('unittestGetTitleOf')
        self.folder.setTitle('the folder')
        self.obj.setTitle('the obj')

        self.setRoles(['Member'])

        response = self.publish('%s/view' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

        output = response.getBody().split(',')
        self.assertEqual(len(output), 4, output)

        self.assertEqual(output, ['the obj', 'the folder', 'the obj', 'the folder'])



import unittest

from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase, atctftestcase

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from Products.Archetypes.interfaces.layer import ILayerContainer
from Products.Archetypes import atapi

from Products.ATContentTypes.content.link import ATLink
from Products.ATContentTypes.interfaces import IATLink
from zope.interface.verify import verifyObject

URL = 'http://www.example.org/'


def editATCT(obj):
    obj.setTitle('Test Title')
    obj.setDescription('Test description')
    obj.setRemoteUrl(URL)

tests = []


class TestSiteATLink(atcttestcase.ATCTTypeTestCase):

    klass = ATLink
    portal_type = 'Link'
    title = 'Link'
    meta_type = 'ATLink'

    def test_implementsATLink(self):
        iface = IATLink
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def testLink(self):
        obj = self._ATCT

        url = 'http://www.example.org/'
        obj.setRemoteUrl(url)
        self.assertEqual(obj.getRemoteUrl(), url)

        url = 'false:url'
        obj.setRemoteUrl(url)
        self.assertEqual(obj.getRemoteUrl(), url)

    def testLinkSanitizesOutput(self):
        obj = self._ATCT

        url = 'javascript:alert("test")'
        obj.setRemoteUrl(url)
        self.assertEqual(obj.getRemoteUrl(),
                             'javascript:alert%28%22test%22%29')
        # Keep question marks and ampersands intact, please.
        url = 'http://something.sane/f.php?p1=value&p2=value'
        obj.setRemoteUrl(url)
        self.assertEqual(obj.getRemoteUrl(),
                             'http://something.sane/f.php?p1=value&p2=value')
        # already quoted values should also remain intact
        url = 'http://something.sane/except with spaces in it'
        expected = 'http://something.sane/except%20with%20spaces%20in%20it'
        obj.setRemoteUrl(url)
        self.assertEqual(obj.getRemoteUrl(), expected)
        obj.setRemoteUrl(obj.getRemoteUrl())
        self.assertEqual(obj.getRemoteUrl(), expected)

    def test_edit(self):
        new = self._ATCT
        editATCT(new)

    def test_get_size(self):
        atct = self._ATCT
        editATCT(atct)
        self.assertEqual(atct.get_size(), len(URL))

tests.append(TestSiteATLink)


class TestATLinkFields(atcttestcase.ATCTFieldTestCase):

    def afterSetUp(self):
        atcttestcase.ATCTFieldTestCase.afterSetUp(self)
        self._dummy = self.createDummy(klass=ATLink)

    def test_remoteUrlField(self):
        dummy = self._dummy
        field = dummy.getField('remoteUrl')

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 1, 'Value is %s' % field.required)
        self.assertTrue(field.default == 'http://', 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == 1, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == (),
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == 0,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getRemoteUrl',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setRemoteUrl',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ModifyPortalContent,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'string', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AttributeStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AttributeStorage(),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.validators == (),
                        'Value is %s' % str(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.StringWidget),
                        'Value is %s' % id(field.widget))
        vocab = field.Vocabulary(dummy)
        self.assertTrue(isinstance(vocab, atapi.DisplayList),
                        'Value is %s' % type(vocab))
        self.assertTrue(tuple(vocab) == (), 'Value is %s' % str(tuple(vocab)))
        self.assertTrue(field.primary == 1, 'Value is %s' % field.primary)

tests.append(TestATLinkFields)


class TestATLinkFunctional(atctftestcase.ATCTIntegrationTestCase):

    portal_type = 'Link'
    views = ('link_view', )

tests.append(TestATLinkFunctional)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



from Products.Archetypes.atapi import BaseSchema
from Products.Archetypes.atapi import MetadataSchema
from Products.Archetypes.atapi import ReferenceField
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import BooleanWidget
from Products.ATContentTypes import ATCTMessageFactory as _
from Products.CMFCore.permissions import ModifyPortalContent

from archetypes.referencebrowserwidget import ReferenceBrowserWidget

# for ATContentTypes we want to have the description in the edit view
# just like CMF
ATContentTypeSchema = BaseSchema.copy() + MetadataSchema((
    BooleanField('excludeFromNav',
        required=False,
        languageIndependent=True,
        schemata='metadata',  # moved to 'default' for folders
        widget=BooleanWidget(
            description=_(u'help_exclude_from_nav', default=u'If selected, this item will not appear in the navigation tree'),
            label=_(u'label_exclude_from_nav', default=u'Exclude from navigation'),
            visible={'view': 'hidden',
                     'edit': 'visible'},
            ),
        ),
    ),)

ATContentTypeSchema['id'].searchable = True
ATContentTypeSchema['id'].validators = ('isValidId',)

# Update the validation layer after change the validator in runtime
ATContentTypeSchema['id']._validationLayer()

ATContentTypeSchema['description'].schemata = 'default'

# BBB
ATContentTypeBaseSchema = ATContentTypeSchema

relatedItemsField = ReferenceField('relatedItems',
        relationship='relatesTo',
        multiValued=True,
        isMetadata=True,
        languageIndependent=False,
        index='KeywordIndex',
        referencesSortable=True,
        keepReferencesOnCopy=True,
        write_permission=ModifyPortalContent,
        widget=ReferenceBrowserWidget(
            allow_search=True,
            allow_browse=True,
            allow_sorting=True,
            show_indexes=False,
            force_close_on_insert=True,
            label=_(u'label_related_items', default=u'Related Items'),
            description='',
            visible={'edit': 'visible', 'view': 'invisible'}
            )
        )
ATContentTypeSchema.addField(relatedItemsField.copy())

#Enabling next / previous navigation

NextPreviousAwareSchema = MetadataSchema((
    BooleanField('nextPreviousEnabled',
        #required=False,
        languageIndependent=True,
        schemata='metadata',
        widget=BooleanWidget(
            description=_(u'help_nextprevious', default=u'This enables next/previous widget on content items contained in this folder.'),
            label=_(u'label_nextprevious', default=u'Enable next previous navigation'),
            visible={'view': 'hidden',
                     'edit': 'visible'},
            ),
        default_method="getNextPreviousParentValue"
        ),
    ),)


def marshall_register(schema):
    try:
        # It's a soft dependency, if not available ignore it.
        from Products.Marshall import ControlledMarshaller
    except ImportError:
        return
    # If it's available, then wrap the existing marshaller with a
    # ControlledMarshaller.
    if not schema.hasLayer('marshall'):
        # We are not interested in schemas that don't have a marshaller.
        return

    # Get existing marshaller.
    marshaller = schema.getLayerImpl('marshall')
    # Check if not already wrapped.
    if isinstance(marshaller, ControlledMarshaller):
        return

    # Wrap into a ControlledMarshaller
    marshaller = ControlledMarshaller(marshaller)
    schema.registerLayer('marshall', marshaller)


def finalizeATCTSchema(schema, folderish=False, moveDiscussion=True):
    """Finalizes an ATCT type schema to alter some fields
    """
    schema.moveField('relatedItems', pos='bottom')
    if folderish:
        schema['relatedItems'].widget.visible['edit'] = 'invisible'
    schema.moveField('excludeFromNav', after='allowDiscussion')
    if moveDiscussion:
        schema.moveField('allowDiscussion', after='relatedItems')

    # Categorization
    if 'subject' in schema:
        schema.changeSchemataForField('subject', 'categorization')
    if 'relatedItems' in schema:
        schema.changeSchemataForField('relatedItems', 'categorization')
    if 'location' in schema:
        schema.changeSchemataForField('location', 'categorization')
    if 'language' in schema:
        schema.changeSchemataForField('language', 'categorization')

    # Dates
    if 'effectiveDate' in schema:
        schema.changeSchemataForField('effectiveDate', 'dates')
    if 'expirationDate' in schema:
        schema.changeSchemataForField('expirationDate', 'dates')
    if 'creation_date' in schema:
        schema.changeSchemataForField('creation_date', 'dates')
    if 'modification_date' in schema:
        schema.changeSchemataForField('modification_date', 'dates')

    # Creators
    if 'creators' in schema:
        schema.changeSchemataForField('creators', 'creators')
    if 'contributors' in schema:
        schema.changeSchemataForField('contributors', 'creators')
    if 'rights' in schema:
        schema.changeSchemataForField('rights', 'creators')

    # Settings
    if 'allowDiscussion' in schema:
        schema.changeSchemataForField('allowDiscussion', 'settings')
    if 'excludeFromNav' in schema:
        schema.changeSchemataForField('excludeFromNav', 'settings')
    if 'nextPreviousEnabled' in schema:
        schema.changeSchemataForField('nextPreviousEnabled', 'settings')

    marshall_register(schema)
    return schema

# Make sure the base ATCT schema is correctly finalized
finalizeATCTSchema(ATContentTypeSchema)

__all__ = ('ATContentTypeSchema', 'relatedItemsField',)



import unittest

from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase

from AccessControl.SecurityManagement import newSecurityManager

from Products.ATContentTypes.lib import constraintypes
from Products.CMFPlone.interfaces import ISelectableConstrainTypes

tests = []


class TestConstrainTypes(atcttestcase.ATCTSiteTestCase):
    folder_type = 'Folder'
    image_type = 'Image'
    document_type = 'Document'
    file_type = 'File'

    def afterSetUp(self):
        atcttestcase.ATCTSiteTestCase.afterSetUp(self)
        self.folder.invokeFactory(self.folder_type, id='af')
        self.tt = self.portal.portal_types
        # an ATCT folder
        self.af = self.folder.af
        # portal_types object for ATCT folder
        self.at = self.tt.getTypeInfo(self.af)

    def test_isMixedIn(self):
        self.assertTrue(isinstance(self.af,
                                   constraintypes.ConstrainTypesMixin),
                        "ConstrainTypesMixin was not mixed in to ATFolder")
        self.assertTrue(ISelectableConstrainTypes.providedBy(self.af),
                        "ISelectableConstrainTypes not implemented by ATFolder instance")

    def test_enabled(self):
        self.af.setConstrainTypesMode(constraintypes.ENABLED)
        self.af.setLocallyAllowedTypes(['Folder', 'Image'])
        self.af.setImmediatelyAddableTypes(['Folder'])

        self.assertEqual(self.af.getLocallyAllowedTypes(),
                                ('Folder', 'Image',))
        self.assertEqual(self.af.getImmediatelyAddableTypes(),
                                ('Folder',))

        self.assertRaises(ValueError, self.af.invokeFactory, 'Document', 'a')
        try:
            self.af.invokeFactory('Image', 'image', title="death")
        except ValueError:
            self.fail()

    def test_disabled(self):
        self.af.setConstrainTypesMode(constraintypes.DISABLED)
        self.af.setLocallyAllowedTypes(['Folder', 'Image'])
        self.af.setImmediatelyAddableTypes(['Folder'])

        # We can still set and persist, even though it is disabled - must
        # remember!
        self.assertEqual(self.af.getRawLocallyAllowedTypes(),
                                ('Folder', 'Image',))
        self.assertEqual(self.af.getRawImmediatelyAddableTypes(),
                                ('Folder',))

        try:
            self.af.invokeFactory('Document', 'whatever', title='life')
            self.af.invokeFactory('Image', 'image', title="more life")
        except ValueError:
            self.fail()

        # Make sure immediately-addable are all types if we are disabled
        allowedIds = [ctype.getId() for ctype in self.af.allowedContentTypes()]
        self.assertEqual(allowedIds, self.af.getImmediatelyAddableTypes())

    def test_acquireFromHomogenousParent(self):
        # Set up outer folder with restrictions enabled
        self.af.setConstrainTypesMode(constraintypes.ENABLED)
        self.af.setLocallyAllowedTypes(['Folder', 'Image'])
        self.af.setImmediatelyAddableTypes(['Folder'])

        # Create inner type to acquire (default)
        self.af.invokeFactory('Folder', 'inner', title='inner')
        inner = self.af.inner

        inner.setConstrainTypesMode(constraintypes.ACQUIRE)

        # Test persistence
        inner.setLocallyAllowedTypes(['Document', 'Event'])
        inner.setImmediatelyAddableTypes(['Document'])

        self.assertEqual(inner.getRawLocallyAllowedTypes(),
                                ('Document', 'Event',))
        self.assertEqual(inner.getRawImmediatelyAddableTypes(),
                                ('Document',))

        self.assertRaises(ValueError, inner.invokeFactory, 'Event', 'a')
        try:
            inner.invokeFactory('Image', 'whatever', title='life')
        except ValueError:
            self.fail()

        # Make sure immediately-addable are inherited
        self.assertEqual(inner.getImmediatelyAddableTypes(),
                         self.af.getImmediatelyAddableTypes())

        # Create a new unprivileged user who can only access the inner folder
        self.portal.acl_users._doAddUser('restricted', 'secret', ['Member'], [])
        inner.manage_addLocalRoles('restricted', ('Manager',))
        # Login the new user
        user = self.portal.acl_users.getUserById('restricted')
        newSecurityManager(None, user)
        self.assertEqual(inner.getLocallyAllowedTypes(),
                        ('Folder', 'Image'))

    def test_acquireFromHetereogenousParent(self):

        # Let folder use a restricted set of types
        self.portal.portal_types.Folder.filter_content_types = 1
        self.portal.portal_types.Folder.allowed_content_types = \
            ('Document', 'Image', 'News Item', 'Topic', 'Folder')

        # Set up outer folder with restrictions enabled
        self.af.setConstrainTypesMode(constraintypes.ENABLED)
        self.af.setLocallyAllowedTypes(['Folder', 'Image'])
        self.af.setImmediatelyAddableTypes(['Folder', 'Image'])

        # Create inner type to acquire (default)
        self.af.invokeFactory('Folder', 'outer', title='outer')
        outer = self.af.outer

        outer.invokeFactory('Folder', 'inner', title='inner')
        inner = outer.inner

        inner.setConstrainTypesMode(constraintypes.ACQUIRE)

        # Test persistence
        inner.setLocallyAllowedTypes(['Document', 'Event'])
        inner.setImmediatelyAddableTypes(['Document'])

        self.assertEqual(inner.getRawLocallyAllowedTypes(),
                                ('Document', 'Event',))
        self.assertEqual(inner.getRawImmediatelyAddableTypes(),
                                ('Document',))

        # Fail - we didn't acquire this, really, since we can't acquire
        # from parent folder of different type
        self.assertRaises(ValueError, inner.invokeFactory, 'Topic', 'a')
        self.assertFalse('News Item' in inner.getLocallyAllowedTypes())

        # Make sure immediately-addable are set to default
        self.assertEqual(inner.getImmediatelyAddableTypes(),
                         inner.getLocallyAllowedTypes())
        self.assertEqual(inner.allowedContentTypes(), outer.allowedContentTypes())

        # Login the new user
        self.portal.acl_users._doAddUser('restricted', 'secret', ['Member'], [])
        inner.manage_addLocalRoles('restricted', ('Manager',))
        user = self.portal.acl_users.getUserById('restricted')
        newSecurityManager(None, user)
        self.assertEqual([t.getId() for t in inner.allowedContentTypes()],
                             ['Folder', 'Image'])


    def test_acquireFromCustomHetereogenousParentWithConstraint(self):
        self.loginAsPortalOwner()
        cp = self.portal.portal_types.manage_copyObjects(['Folder'])
        self.portal.portal_types.manage_pasteObjects(cp)
        self.portal.portal_types.manage_renameObject('copy_of_Folder', 'Folder2')

        # Let folder use a restricted set of types
        self.portal.portal_types.Folder.filter_content_types = 1
        self.portal.portal_types.Folder.allowed_content_types = \
            ('Document', 'Image', 'News Item', 'Topic', 'Folder', 'Folder2')

        self.portal.portal_types.Folder2.filter_content_types = 1
        self.portal.portal_types.Folder2.allowed_content_types = \
            ('Document', 'Image', 'Topic', 'Folder')

        # Set up outer folder with restrictions enabled
        self.af.setConstrainTypesMode(constraintypes.ENABLED)
        self.af.setLocallyAllowedTypes(['Folder2', 'Image', 'News Item'])
        self.af.setImmediatelyAddableTypes(['Folder2', 'Image'])

        # Create type to acquire
        self.af.invokeFactory('Folder2', 'folder2', title='folder2')
        folder2 = self.af.folder2
        folder2.setConstrainTypesMode(constraintypes.ACQUIRE)

        # News item is not in addable types because it is globally forbidden in Folder2 type
        # and Folder is not in addable types because it is locally forbidden in folder2 parent
        self.assertEqual([fti.getId() for fti in folder2.allowedContentTypes()],
                             ['Image'])
        self.assertEqual(folder2.getImmediatelyAddableTypes(), ['Image'])

        # Login the new user
        self.portal.acl_users._doAddUser('restricted', 'secret', ['Member'], [])
        folder2.manage_addLocalRoles('restricted', ('Manager',))
        user = self.portal.acl_users.getUserById('restricted')
        newSecurityManager(None, user)
        self.assertEqual([t.getId() for t in self.af.allowedContentTypes()],
                             [])
        self.assertEqual([t.getId() for t in folder2.allowedContentTypes()],
                             ['Image'])


    def test_acquireFromCustomHetereogenousParentWithoutConstraint(self):
        self.loginAsPortalOwner()
        cp = self.portal.portal_types.manage_copyObjects(['Folder'])
        self.portal.portal_types.manage_pasteObjects(cp)
        self.portal.portal_types.manage_renameObject('copy_of_Folder', 'Folder2')

        # Let folder use a restricted set of types
        self.portal.portal_types.Folder.filter_content_types = 1
        self.portal.portal_types.Folder.allowed_content_types = \
            ('Document', 'Image', 'News Item', 'Topic', 'Folder', 'Folder2')

        self.portal.portal_types.Folder2.filter_content_types = 1
        self.portal.portal_types.Folder2.allowed_content_types = \
            ('Document', 'Image', 'Topic', 'Folder')

        # Set up outer folder with restrictions disabled
        self.af.setConstrainTypesMode(constraintypes.DISABLED)

        # Create type to acquire
        self.af.invokeFactory('Folder2', 'folder2', title='folder2')
        folder2 = self.af.folder2
        folder2.setConstrainTypesMode(constraintypes.ACQUIRE)

        # News item is not in addable types because it is globally forbidden in Folder2 type
        # and Folder is not in addable types because it is locally forbidden in folder2 parent
        self.assertEqual([fti.getId() for fti in folder2.allowedContentTypes()],
                             ['Document', 'Folder', 'Image', 'Topic'])
        self.assertEqual(folder2.getImmediatelyAddableTypes(),
                             ['Document', 'Folder', 'Image', 'Topic'])

        # Login the new user
        self.portal.acl_users._doAddUser('restricted', 'secret', ['Member'], [])
        folder2.manage_addLocalRoles('restricted', ('Manager',))
        user = self.portal.acl_users.getUserById('restricted')
        newSecurityManager(None, user)
        self.assertEqual([t.getId() for t in self.af.allowedContentTypes()],
                             [])
        self.assertEqual([t.getId() for t in folder2.allowedContentTypes()],
                             ['Document', 'Folder', 'Image', 'Topic'])

tests.append(TestConstrainTypes)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



import unittest

from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase

from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.config import TOOLNAME
from Products.ATContentTypes.interfaces import IATCTTopicsTool
from zope.interface.verify import verifyObject

tests = []
index_def = {'index': 'end',
             'friendlyName': 'End Date For Test',
             'description': 'This is an end Date',
             'criteria': ['ATDateCriteria', 'ATDateRangeCriteria']
            }
meta_def = {'metadata': 'ModificationDate',
            'friendlyName': 'Modification Date For Test',
            'description': ''
           }


class TestTool(atcttestcase.ATCTSiteTestCase):

    def afterSetUp(self):
        self.tool = getattr(self.portal.aq_explicit, TOOLNAME)

    def test_interface(self):
        self.assertTrue(IATCTTopicsTool.providedBy(self.tool))
        self.assertTrue(verifyObject(IATCTTopicsTool, self.tool))

    #Index tests
    def test_add_index(self):
        t = self.tool
        t.addIndex(enabled=True, **index_def)
        index = t.getIndex(index_def['index'])
        self.assertEqual(index.index, index_def['index'])
        self.assertEqual(index.friendlyName, index_def['friendlyName'])
        self.assertEqual(index.description, index_def['description'])
        # Only need to test truth not actual value
        self.assertTrue(index.enabled)
        self.assertEqual(index.criteria, tuple(index_def['criteria']))

        self.assertTrue(index in t.getEnabledIndexes())
        self.assertTrue(index_def['index'] in [a[0] for a in t.getEnabledFields()])
        self.assertTrue(index_def['index'] in t.getIndexDisplay(True).keys())
        self.assertTrue(index_def['friendlyName'] in t.getIndexDisplay(True).values())
        self.assertTrue(index_def['index'] in t.getIndexes(1))

    def test_disable_index(self):
        t = self.tool
        t.addIndex(enabled=False, **index_def)
        index = t.getIndex(index_def['index'])
        self.assertEqual(index.index, index_def['index'])
        self.assertEqual(index.friendlyName, index_def['friendlyName'])
        self.assertEqual(index.description, index_def['description'])
        # Only need to test truth not actual value
        self.assertFalse(index.enabled)
        self.assertEqual(index.criteria, tuple(index_def['criteria']))

        self.assertFalse(index in t.getEnabledIndexes())
        self.assertFalse(index_def['index'] in [a[0] for a in t.getEnabledFields()])
        self.assertFalse(index_def['index'] in t.getIndexes(1))
        self.assertFalse(index_def['index'] in t.getIndexDisplay(True).keys())
        self.assertTrue(index_def['friendlyName'] not in t.getIndexDisplay(True).values())
        # Make sure it's still in the un-limited list
        self.assertTrue(index_def['index'] in t.getIndexDisplay(False).keys())
        self.assertTrue(index_def['friendlyName'] in t.getIndexDisplay(False).values())
        self.assertTrue(index_def['index'] in t.getIndexes())

    def test_add_bogus_index(self):
        # You can add metadata that's not in the catalog
        t = self.tool
        t.addIndex('bogosity', enabled=True)
        self.assertTrue(t.getIndex('bogosity'))

        #Add
        t.addIndex('bogosity', enabled=True)
        self.assertTrue('bogosity' in [a[0] for a in t.getEnabledFields()])
        #Add
        t.addIndex('bogosity', enabled=True)
        self.assertTrue('bogosity' in t.getIndexDisplay(True).keys())
        #Add
        t.addIndex('bogosity', enabled=True)
        self.assertTrue('bogosity' in t.getIndexes(1))
        #Add
        t.addIndex('bogosity', enabled=True)
        self.assertTrue('bogosity' in [i.index for i in t.getEnabledIndexes()])

    def test_remove_index(self):
        t = self.tool
        t.addIndex(**index_def)
        t.removeIndex(index_def['index'])
        error = None
        try:
            index = t.topic_indexes[index_def['index']]
        except KeyError:
            error = True
        self.assertTrue(error)

        error = True
        try:
            index = t.getIndex(index_def['index'])
        except AttributeError:
            error = False
        self.assertFalse(error)

    def test_update_index(self):
        # An index with no criteria set should set all available criteria,
        # also changes made using updateIndex should not reset already set
        # values
        t = self.tool
        t.addIndex(enabled=True, **index_def)
        t.updateIndex(index_def['index'], criteria=None,
                      description='New Description')
        index = t.getIndex(index_def['index'])
        self.assertTrue(index.criteria)
        self.assertTrue(index.criteria != index_def['criteria'])
        self.assertTrue(index.description == 'New Description')
        self.assertTrue(index.friendlyName == index_def['friendlyName'])
        self.assertTrue(index.enabled)

    def test_all_indexes(self):
        # Ensure that the tool includes all indexes in the catalog
        t = self.tool
        cat = getToolByName(self.tool, 'portal_catalog')
        indexes = [field for field in cat.indexes()]
        init_indexes = list(t.getIndexes())
        unique_indexes = [i for i in indexes if i not in init_indexes]
        unique_indexes = unique_indexes + [i for i in init_indexes if i not in indexes]
        self.assertFalse(unique_indexes)

    def test_change_catalog_index(self):
        t = self.tool
        cat = getToolByName(self.tool, 'portal_catalog')
        #add
        error = True
        cat.manage_addIndex('nonsense', 'FieldIndex')
        try:
            t.getIndex('nonsense')
        except AttributeError:
            error = False
        self.assertFalse(error)
        #remove
        error = False
        cat.delIndex('nonsense')
        try:
            t.getIndex('nonsense')
        except AttributeError:
            error = True
        self.assertTrue(error)

    #Metadata tests
    def test_add_metadata(self):
        t = self.tool
        t.addMetadata(enabled=True, **meta_def)
        meta = t.getMetadata(meta_def['metadata'])
        self.assertEqual(meta.index, meta_def['metadata'])
        self.assertEqual(meta.friendlyName, meta_def['friendlyName'])
        self.assertEqual(meta.description, meta_def['description'])
        # Only need to test truth not actual value
        self.assertTrue(meta.enabled)

        self.assertTrue(meta in t.getEnabledMetadata())
        self.assertTrue(meta_def['metadata'] in t.getMetadataDisplay(True).keys())
        self.assertTrue(meta_def['friendlyName'] in t.getMetadataDisplay(True).values())
        self.assertTrue(meta_def['metadata'] in t.getAllMetadata(1))

    def test_disable_metadata(self):
        t = self.tool
        t.addMetadata(enabled=False, **meta_def)
        meta = t.getMetadata(meta_def['metadata'])
        self.assertEqual(meta.index, meta_def['metadata'])
        self.assertEqual(meta.friendlyName, meta_def['friendlyName'])
        self.assertEqual(meta.description, meta_def['description'])
        # Only need to test truth not actual value
        self.assertFalse(meta.enabled)

        self.assertTrue(meta not in t.getEnabledMetadata())
        self.assertFalse(meta_def['metadata'] in t.getAllMetadata(1))
        self.assertFalse(meta_def['metadata'] in t.getMetadataDisplay(True).keys())
        self.assertFalse(meta_def['friendlyName'] in t.getMetadataDisplay(True).values())
        # Make sure it's still in the un-limited list
        self.assertTrue(meta_def['metadata'] in t.getMetadataDisplay(False).keys())
        self.assertTrue(meta_def['friendlyName'] in t.getMetadataDisplay(False).values())
        self.assertTrue(meta_def['metadata'] in t.getAllMetadata())

    def test_add_bogus_metadata(self):
        # You can add metdata that's not in the catalog
        t = self.tool
        t.addMetadata('bogosity', enabled=True)
        self.assertTrue(t.getMetadata('bogosity'))

        #Add
        t.addMetadata('bogosity', enabled=True)
        self.assertTrue('bogosity' in t.getMetadataDisplay(True).keys())
        #Add
        t.addMetadata('bogosity', enabled=True)
        self.assertTrue('bogosity' in t.getAllMetadata(1))
        #Add
        t.addMetadata('bogosity', enabled=True)
        self.assertTrue('bogosity' in [i.index for i in t.getEnabledMetadata()])

    def test_remove_metadata(self):
        t = self.tool
        t.addMetadata(**meta_def)
        t.removeMetadata(meta_def['metadata'])
        error = None
        try:
            meta = t.topic_metadata[meta_def['metadata']]
        except KeyError:
            error = True
        self.assertTrue(error)

        error = True
        try:
            meta = t.getMetadata(meta_def['metadata'])
        except AttributeError:
            error = False
        self.assertFalse(error)

    def test_update_metadata(self):
        # Changes made using updateMetadata should not reset already set
        # values
        t = self.tool
        t.addMetadata(enabled=True, **meta_def)
        t.updateMetadata(meta_def['metadata'], friendlyName='New Name')
        meta = t.getMetadata(meta_def['metadata'])
        self.assertTrue(meta.friendlyName == 'New Name')
        self.assertTrue(meta.enabled)

    def test_all_metadata(self):
        # Ensure that the tool includes all metadata in the catalog
        t = self.tool
        cat = getToolByName(self.tool, 'portal_catalog')
        metadata = [field for field in cat.schema()]
        init_metadata = list(t.getAllMetadata())
        unique_metadata = [i for i in metadata if i not in init_metadata]
        unique_metadata = unique_metadata + [i for i in init_metadata if i not in metadata]
        self.assertFalse(unique_metadata)

    def test_change_catalog_schema(self):
        t = self.tool
        cat = getToolByName(self.tool, 'portal_catalog')
        #add
        error = True
        cat.manage_addColumn('nonsense')
        try:
            t.getMetadata('nonsense')
        except AttributeError:
            error = False
        self.assertFalse(error)
        #remove
        error = False
        cat.delColumn('nonsense')
        try:
            t.getMetadata('nonsense')
        except AttributeError:
            error = True
        self.assertTrue(error)

tests.append(TestTool)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo
from zope.interface import implements

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import BooleanWidget

from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import SORT_INDICES
from Products.ATContentTypes.interfaces import IATTopicSortCriterion
from Products.ATContentTypes.permission import ChangeTopics
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema

from Products.ATContentTypes import ATCTMessageFactory as _

ATSortCriterionSchema = ATBaseCriterionSchema + Schema((
    BooleanField('reversed',
                required=0,
                mode="rw",
                write_permission=ChangeTopics,
                default=0,
                widget=BooleanWidget(label=_(u'Reverse')),
                ),

    ))


class ATSortCriterion(ATBaseCriterion):
    """A sort criterion"""

    implements(IATTopicSortCriterion)
    security = ClassSecurityInfo()
    schema = ATSortCriterionSchema
    meta_type = 'ATSortCriterion'
    archetype_name = 'Sort Criterion'
    shortDesc = 'Sort'

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        result = [('sort_on', self.Field())]

        if self.getReversed():
            result.append(('sort_order', 'reverse'))

        return tuple(result)

registerCriterion(ATSortCriterion, SORT_INDICES)



import unittest

from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase, atctftestcase
from Products.ATContentTypes.tests.utils import dcEdit

from Products.CMFCore.permissions import View
from Products.Archetypes.interfaces.layer import ILayerContainer
from Products.Archetypes import atapi

from Products.Archetypes.Field import BooleanField
from Products.CMFCore.utils import getToolByName

from Products.ATContentTypes.permission import ChangeTopics
from Products.ATContentTypes.content.topic import ATTopic
from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.tests.utils import EmptyValidator
from Products.ATContentTypes.interfaces import IATTopic
from zope.interface.verify import verifyObject
from OFS.interfaces import IOrderedContainer

from Products.CMFPlone.PloneBatch import Batch


ACQUIRE = True
LIMIT = False
COUNT = '150'
CUSTOM = True
FIELDS = ('start', 'end', 'Creator')

CRITERIA_SETUP = {'Integer Criterion':  # Meta Type
                        ('portal_type',  # Field
                         '10 10',  # Value
                         'min:max'),  # Direction
                  'String Criterion':
                        ('SearchableText',
                         'portal'),
                  'Friendly Date Criterion':
                        ('start',
                         '10',
                         'within_day',  # Operation
                         'ahead'),  # DateRange
                  'List Criterion':
                        ('Subject',
                         "value1\nportal\ntest",
                         'OR'),  # Operator
                  'Sort Criterion':
                        ('getId',
                         False),  # Reversed
                }

CRIT_MAP = {'Integer Criterion': 'ATSimpleIntCriterion',
            'String Criterion': 'ATSimpleStringCriterion',
            'Friendly Date Criterion': 'ATFriendlyDateCriteria',
            'List Criterion': 'ATListCriterion',
            'Sort Criterion': 'ATSortCriterion'}

REV_CRIT_MAP = dict([[v, k] for k, v in CRIT_MAP.items()])


def editATCT(obj):
    dcEdit(obj)
    obj.setAcquireCriteria(ACQUIRE)
    obj.setLimitNumber(LIMIT)
    obj.setItemCount(COUNT)
    #obj.setCustomView(CUSTOM)
    #obj.setCustomViewFields(FIELDS)
    for meta in CRITERIA_SETUP.keys():
        AT_META = CRIT_MAP[meta]
        CRIT_FIELD = CRITERIA_SETUP[meta][0]
        obj.addCriterion(CRIT_FIELD, AT_META)
    for crit in obj.listCriteria():
        CRIT_TYPE = crit.meta_type
        OLD_CRIT_TYPE = REV_CRIT_MAP[CRIT_TYPE]
        params = CRITERIA_SETUP[OLD_CRIT_TYPE][1:]
        if CRIT_TYPE not in ['ATSortCriterion', 'ATSimpleIntCriterion']:
            crit.setValue(params[0])
        if CRIT_TYPE == 'ATFriendlyDateCriteria':
            crit.setOperation(params[1])
            DATE_RANGE = (params[2] == 'ahead' and '+') or '-'
            crit.setDateRange(DATE_RANGE)
        if CRIT_TYPE == 'ATListCriterion':
            crit.setOperator(params[1])
        if CRIT_TYPE == 'ATSimpleIntCriterion':
            value = params[0].split(' ')
            crit.setValue(value[0])
            if len(value) > 1:
                crit.setValue2(value[1])
            crit.setDirection(params[1])
        if CRIT_TYPE == 'ATSortCriterion':
            crit.setReversed(params[0])


def convert_old_catalog_usage(criteria_items):
    """Convert old style query parameters into records"""
    if len(criteria_items) > 1:
        field = criteria_items[0][0]
        query_val = criteria_items[0][1]
        extra_param = criteria_items[1][1]
        if '_usage' in criteria_items[1][0]:
            usage = extra_param.split(':')
            extra_type = usage[0].strip()
            extra_param = ':'.join(usage[1:]).strip()
        else:
            extra_type = criteria_items[1][0].replace('%s_' % field, '')
        criteria_items = [(field, {'query': query_val, extra_type: extra_param})]
    return tuple(criteria_items)


def convert_old_catalog_query(query):
    """ Convert old style query to new record based query """
    for k, v in query.items():
        q_field = q_type = q_param = None
        if '_usage' in k:
            q_field = k.replace('_usage', '')
            usage = v.split(':')
            q_type = usage[0].strip()
            q_param = ':'.join(usage[1:]).strip()
        elif '_operator' in k:
            q_field = k.replace('_operator', '')
            q_type = 'operator'
            q_param = v
        if q_field:
            new_val = query[q_field]
            if not isinstance(v, dict):
                new_val = {'query': new_val}
            new_val[q_type] = q_param
            query[q_field] = new_val
            del query[k]
    return query

tests = []


class TestSiteATTopic(atcttestcase.ATCTTypeTestCase):

    klass = ATTopic
    portal_type = 'Topic'
    title = 'Collection'
    meta_type = 'ATTopic'

    def afterSetUp(self):
        self.setRoles(['Manager', 'Member'])
        self._ATCT = self._createType(self.folder, self.portal_type, 'ATCT')
        self.setRoles(['Member'])

    def test_typeInfo(self):
        ti = self._ATCT.getTypeInfo()
        self.assertEqual(ti.getId(), self.portal_type)
        self.assertTrue(ti.Title().startswith('Collection'))

    def test_implementsATTopic(self):
        iface = IATTopic
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_isNotOrdered(self):
        iface = IOrderedContainer
        self.assertFalse(iface.providedBy(self._ATCT))
        self.assertFalse(iface.providedBy(self.klass))

    def test_Empty(self):
        topic = self._ATCT

        query = topic.buildQuery()
        self.assertEqual(query, None)

    def test_canContainSubtopics(self):
        ttool = self.portal.portal_types
        fti = ttool.getTypeInfo(self.portal_type)
        self.assertTrue(self.portal_type not in fti.allowed_content_types,
                        'Topics should not be allowed to contain topics')

    def test_Simple(self):
        topic = self._ATCT

        topic.addCriterion('foo', 'ATSimpleStringCriterion')
        self.assertTrue('crit__foo_ATSimpleStringCriterion' in topic)
        topic.getCriterion('foo_ATSimpleStringCriterion').setValue('bar')

        query = topic.buildQuery()
        self.assertEqual(len(query), 1)
        self.assertEqual(query['foo'], 'bar')

        topic.addCriterion('baz', 'ATSimpleIntCriterion')
        topic.getCriterion('baz_ATSimpleIntCriterion').setValue('43')

        query = topic.buildQuery()
        self.assertEqual(len(query), 2)
        self.assertEqual(query['foo'], 'bar')
        self.assertEqual(query['baz'], {'query': 43})

    def test_nested(self):
        topic = self._ATCT

        topic.addCriterion('foo', 'ATSimpleStringCriterion')
        self.assertTrue('crit__foo_ATSimpleStringCriterion' in topic)
        topic.getCriterion('foo_ATSimpleStringCriterion').setValue('bar')

        self.setRoles(['Manager', 'Member'])
        topic.addSubtopic('qux')
        self.setRoles(['Member'])
        subtopic = topic.qux

        subtopic.setAcquireCriteria(True)

        #Ensure an empty subtopic uses it's parents' queries
        self.assertEqual(subtopic.buildQuery(), topic.buildQuery())

        subtopic.addCriterion('baz', 'ATSimpleStringCriterion')
        self.assertTrue('crit__baz_ATSimpleStringCriterion' in subtopic)
        subtopic.getCriterion('baz_ATSimpleStringCriterion').setValue('bam')

        query = subtopic.buildQuery()
        self.assertEqual(len(query), 2)
        self.assertEqual(query['foo'], 'bar')
        self.assertEqual(query['baz'], 'bam')

        subtopic.setAcquireCriteria(False)
        query = subtopic.buildQuery()
        self.assertEqual(len(query), 1)
        self.assertEqual(query['baz'], 'bam')

    def test_nested_friendly_date_criteria(self):
        """
        The queries before adding fix for https://dev.plone.org/plone/ticket/8827
        where subtopics should inhert start / end keys
        topic query: {
        'start': {'query': DateTime('2009/01/30 21:54:27.370 GMT+1'), 'range': 'min'}
        }
        subtopic query: {
        'start': {'query': DateTime('2009/01/30 21:54:27.444 GMT+1'), 'range': 'min'},
        'end': {'query': DateTime('2009/01/30 21:54:27.445 GMT+1'), 'range': 'max'}
        }
        ^^ the 'start' key in the subtopic query is odd and results in combination
        with the 'end' key in zero results even if there is old/past items.
        """
        # Add topic - future items
        topic = self._ATCT
        date_crit = topic.addCriterion('start', 'ATFriendlyDateCriteria')
        date_crit.setValue(0)
        date_crit.setDateRange('+')  # This is irrelevant when the date is now
        date_crit.setOperation('more')
        # Add subtopic - past items
        self.setRoles(['Manager', 'Member'])
        topic.addSubtopic('qux')
        self.setRoles(['Member'])
        subtopic = topic.qux
        date_crit = subtopic.addCriterion('end', 'ATFriendlyDateCriteria')
        date_crit.setValue(0)
        date_crit.setDateRange('-')  # This is irrelevant when the date is now
        date_crit.setOperation('less')
        subtopic.setAcquireCriteria(True)
        # fetch the query
        query = subtopic.buildQuery()
        self.assertTrue(query['end'])
        # query shouldn't have a start key https://dev.plone.org/plone/ticket/8827
        self.assertFalse('start' in query)

    def test_nested_friendly_date_criteria_reverse(self):
        """
        Lets have a test for the reverse situation
        when the main topic lists past items and
        the subtopics lists furture items.
        """
        # Add topic - past items
        topic = self._ATCT
        date_crit = topic.addCriterion('end', 'ATFriendlyDateCriteria')
        date_crit.setValue(0)
        date_crit.setDateRange('+')  # This is irrelevant when the date is now
        date_crit.setOperation('less')
        # Add subtopic - future items
        self.setRoles(['Manager', 'Member'])
        topic.addSubtopic('qux')
        self.setRoles(['Member'])
        subtopic = topic.qux
        date_crit = subtopic.addCriterion('start', 'ATFriendlyDateCriteria')
        date_crit.setValue(0)
        date_crit.setDateRange('-')  # This is irrelevant when the date is now
        date_crit.setOperation('more')
        subtopic.setAcquireCriteria(True)
        # fetch the query
        query = subtopic.buildQuery()
        # this one can have both start and end
        self.assertTrue(query['start'])
        self.assertTrue(query['end'])

    def test_nested_friendly_date_criteria_with_a_start(self):
        """
        Use case: A subtopic with past item but only certain days back
        in time for example 1 month.
        Lets ensure the 'start' isn't wiped out of the query
        in this use case
        """
        # Add topic - future items
        topic = self._ATCT
        date_crit = topic.addCriterion('start', 'ATFriendlyDateCriteria')
        date_crit.setValue(0)
        date_crit.setDateRange('+')  # This is irrelevant when the date is now
        date_crit.setOperation('more')
        # Add subtopic - past items - but only 1 month back in time
        self.setRoles(['Manager', 'Member'])
        topic.addSubtopic('qux')
        self.setRoles(['Member'])
        subtopic = topic.qux
        date_crit = subtopic.addCriterion('end', 'ATFriendlyDateCriteria')
        date_crit.setValue(0)
        date_crit.setDateRange('-')  # This is irrelevant when the date is now
        date_crit.setOperation('less')
        subtopic.setAcquireCriteria(True)
        date_crit = subtopic.addCriterion('start', 'ATFriendlyDateCriteria')
        date_crit.setValue(31)  # 1 month
        date_crit.setDateRange('-')  # the opposite is marked '+'
        date_crit.setOperation('less')  # the opposite is marked 'more'
        #not sur about this one
        subtopic.setAcquireCriteria(True)
        # fetch the query
        query = subtopic.buildQuery()
        self.assertTrue(query['end'])
        # query should have a start key
        self.assertTrue(query['start'])

    def test_edit(self):
        new = self._ATCT
        editATCT(new)

    def test_queryCatalogBatching(self):
        # Ensure that has we return a proper batch if requested
        topic = self._ATCT
        self.assertTrue(isinstance(topic.queryCatalog(batch=True), Batch))
        self.assertFalse(isinstance(topic.queryCatalog(), Batch))
        # try it with some content now
        crit = topic.addCriterion('portal_type', 'ATSimpleStringCriterion')
        crit.setValue('Folder')
        self.assertTrue(isinstance(topic.queryCatalog(batch=True), Batch))
        self.assertFalse(isinstance(topic.queryCatalog(), Batch))

    def test_queryCatalogBatchingWithLimit(self):
        # Ensure that the number of results is the same with or without a
        # limit
        topic = self._ATCT
        crit = topic.addCriterion('portal_type', 'ATSimpleStringCriterion')
        crit.setValue('Folder')
        # set a sort criterion because sort_limit affects result batching.
        topic.setSortCriterion('created', False)
        # add a few folders
        for i in range(6):
            self.folder.invokeFactory('Folder', 'folder_%s' % i)
            getattr(self.folder, 'folder_%s' % i).reindexObject()
        num_items = len(topic.queryCatalog())
        # We better have some folders
        self.assertTrue(num_items >= 6)
        self.assertEqual(topic.queryCatalog(batch=True).sequence_length, num_items)
        # Set some limits
        topic.setLimitNumber(True)
        topic.setItemCount(2)
        self.assertEqual(topic.queryCatalog(batch=True).sequence_length, num_items)

    def test_queryCatalogBrains(self):
        #Ensure that we feturn full objects when requested
        topic = self._ATCT
        crit = topic.addCriterion('portal_type', 'ATSimpleStringCriterion')
        crit.setValue('Folder')
        self.assertTrue(isinstance(topic.queryCatalog(full_objects=True)[0], ATFolder))
        self.assertFalse(isinstance(topic.queryCatalog()[0], ATFolder))

    def test_queryCatalogLimitChangesBatchSize(self):
        #Ensure that a set limit overrides batch size
        topic = self._ATCT
        topic.setLimitNumber(True)
        topic.setItemCount(10)
        crit = topic.addCriterion('portal_type', 'ATSimpleStringCriterion')
        crit.setValue('Folder')
        # Add a bunch of folders.
        for i in range(1, 20):
            self.folder.invokeFactory('Folder', str(i))
        self.assertTrue(isinstance(topic.queryCatalog(batch=True), Batch))
        # Check the batch length
        self.assertEqual(len(topic.queryCatalog(batch=True)), 10)

    def test_queryCatalogBSizeChangesBatchSize(self):
        #Ensure that a set limit overrides batch size
        topic = self._ATCT
        crit = topic.addCriterion('portal_type', 'ATSimpleStringCriterion')
        crit.setValue('Folder')
        # Add a bunch of folders.
        for i in range(1, 20):
            self.folder.invokeFactory('Folder', str(i))
        self.assertTrue(isinstance(topic.queryCatalog(batch=True, b_size=5), Batch))
        # Check the batch length
        self.assertEqual(len(topic.queryCatalog(batch=True, b_size=5)), 5)

    def test_queryCatalogAddCriteria(self):
        #Ensure that we can add params
        topic = self._ATCT
        crit = topic.addCriterion('portal_type', 'ATSimpleStringCriterion')
        crit.setValue('Folder')
        # Add a bunch of folders.
        for i in range(1, 20):
            self.folder.invokeFactory('Folder', str(i))
        self.assertEqual(len(topic.queryCatalog(sort_on='Date', sort_limit=5)), 5)

    def test_queryCatalogOverrideCriteria(self):
        #Ensure that we can override params
        topic = self._ATCT
        crit = topic.addCriterion('portal_type', 'ATSimpleStringCriterion')
        crit.setValue('Document')
        # Add a bunch of folders.
        for i in range(1, 20):
            self.folder.invokeFactory('Folder', str(i))
        self.assertEqual(len(topic.queryCatalog(portal_type='Folder')), 23)

    def test_get_size(self):
        atct = self._ATCT
        self.assertEqual(atct.get_size(), 1)

    def test_syndication_enabled_by_default(self):
        syn = getToolByName(self.portal, 'portal_syndication')
        self.assertTrue(syn.isSyndicationAllowed(self._ATCT))

    def test_schema_marshall(self):
        pass

    def test_sort_criterion_does_not_affect_available_fields(self):
        topic = self._ATCT
        # set a sort criterion
        topic.setSortCriterion('created', False)
        # It should still be available for other criteria
        self.assertTrue([i for i in topic.listAvailableFields()
                         if i[0] == 'created'])
        # Add a normal criteria for the same field
        crit = topic.addCriterion('created', 'ATFriendlyDateCriteria')
        # It should no longer be available
        self.assertFalse([i for i in topic.listAvailableFields()
                     if i[0] == 'created'])

    def test_album_images_collection(self):
        # album view of a collection of Image objects display images in 'images' section
        portal = self.portal
        self.loginAsPortalOwner()
        portal.invokeFactory('Image', 'image1')
        portal.invokeFactory('Image', 'image2')
        portal.invokeFactory('Image', 'image3')

        topic = self._ATCT

        crit = topic.addCriterion('Type', 'ATSimpleStringCriterion')
        crit.setValue('Image')
        album = topic.atctListAlbum(images=1, folders=1, others=1)
        self.assertEqual(len(album['folders']), 0)
        self.assertEqual(len(album['images']), 3)
        self.assertEqual(len(album['others']), 0)

tests.append(TestSiteATTopic)


class TestATTopicFields(atcttestcase.ATCTFieldTestCase):

    def afterSetUp(self):
        atcttestcase.ATCTFieldTestCase.afterSetUp(self)
        self._dummy = self.createDummy(klass=ATTopic)

    def test_acquireCriteriaField(self):
        dummy = self._dummy
        field = dummy.getField('acquireCriteria')
        field_vocab = BooleanField._properties.get('vocabulary', ())

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 0, 'Value is %s' % field.required)
        self.assertTrue(field.default == False, 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == 0, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == field_vocab,
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == 0,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getAcquireCriteria',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setAcquireCriteria',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ChangeTopics,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'boolean', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AttributeStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AttributeStorage(),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(field.validators == EmptyValidator,
                        'Value is %s' % str(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.BooleanWidget),
                        'Value is %s' % id(field.widget))
        vocab = field.Vocabulary(dummy)
        self.assertTrue(isinstance(vocab, atapi.DisplayList),
                        'Value is %s' % type(vocab))
        self.assertTrue(tuple(vocab) == tuple([x[0] for x in field_vocab]),
                        'Value is %s' % str(tuple(vocab)))

    def test_limitNumberField(self):
        dummy = self._dummy
        field = dummy.getField('limitNumber')
        field_vocab = BooleanField._properties.get('vocabulary', ())

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 0, 'Value is %s' % field.required)
        self.assertTrue(field.default == False, 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == 0, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == field_vocab,
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == 0,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getLimitNumber',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setLimitNumber',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ChangeTopics,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'boolean', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AttributeStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AttributeStorage(),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(field.validators == EmptyValidator,
                        'Value is %s' % str(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.BooleanWidget),
                        'Value is %s' % id(field.widget))
        vocab = field.Vocabulary(dummy)
        self.assertTrue(isinstance(vocab, atapi.DisplayList),
                        'Value is %s' % type(vocab))
        self.assertTrue(tuple(vocab) == tuple([x[0] for x in field_vocab]),
                        'Value is %s' % str(tuple(vocab)))

    def test_itemCountField(self):
        dummy = self._dummy
        field = dummy.getField('itemCount')

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 0, 'Value is %s' % field.required)
        self.assertTrue(field.default == 0, 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == 0, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == (),
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == 0,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getItemCount',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setItemCount',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ChangeTopics,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'integer', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AttributeStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AttributeStorage(),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(field.validators == EmptyValidator,
                        'Value is %s' % str(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.IntegerWidget),
                        'Value is %s' % id(field.widget))
        vocab = field.Vocabulary(dummy)
        self.assertTrue(isinstance(vocab, atapi.DisplayList),
                        'Value is %s' % type(vocab))
        self.assertTrue(tuple(vocab) == (), 'Value is %s' % str(tuple(vocab)))

    def test_customViewField(self):
        # XXX not in the current version
        return
        dummy = self._dummy
        field = dummy.getField('customView')

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 0, 'Value is %s' % field.required)
        self.assertTrue(field.default == False, 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == 0, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == (),
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == 0,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getCustomView',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setCustomView',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ChangeTopics,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'boolean', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AttributeStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AttributeStorage(),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(field.validators == EmptyValidator,
                        'Value is %s' % str(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.BooleanWidget),
                        'Value is %s' % id(field.widget))
        vocab = field.Vocabulary(dummy)
        self.assertTrue(isinstance(vocab, atapi.DisplayList),
                        'Value is %s' % type(vocab))
        self.assertTrue(tuple(vocab) == (), 'Value is %s' % str(tuple(vocab)))

    def test_customViewFieldsField(self):
        # XXX not in the current version
        return
        dummy = self._dummy
        field = dummy.getField('customViewFields')

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 0, 'Value is %s' % field.required)
        self.assertTrue(field.default == ('Title',), 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == 0, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == 'listMetaDataFields',
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == True,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getCustomViewFields',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setCustomViewFields',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ChangeTopics,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'lines', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AttributeStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AttributeStorage(),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(field.validators == EmptyValidator,
                        'Value is %s' % str(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.InAndOutWidget),
                        'Value is %s' % id(field.widget))

tests.append(TestATTopicFields)


class TestATTopicFunctional(atctftestcase.ATCTIntegrationTestCase):

    def afterSetUp(self):
        # adding topics is restricted
        self.setRoles(['Manager', 'Member'])
        atctftestcase.ATCTIntegrationTestCase.afterSetUp(self)

    def test_dynamic_view_without_view(self):
        # dynamic view magic should work
        response = self.publish('%s/' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)

    portal_type = 'Topic'
    views = ('atct_topic_view', 'criterion_edit_form', 'atct_topic_subtopics')

tests.append(TestATTopicFunctional)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



from plone.memoize import ram
from zope.publisher.browser import BrowserView

from Products.CMFCore.utils import getToolByName

from Products.ATContentTypes.interfaces import ICalendarSupport
from Products.ATContentTypes.lib import calendarsupport as cs


def cachekey(fun, self):
    """ generate a cache key based on the following data:
          * context URL
          * fingerprint of the brains found in the query
        the returned key is suitable for usage with `memoize.ram.cache` """
    context = self.context
    def add(brain):
        path = brain.getPath().decode('ascii', 'replace').encode('utf-8')
        return '%s\n%s\n\n' % (path, brain.modified)
    url = context.absolute_url()
    title = context.Title()
    fingerprint = ''.join(map(add, self.events))
    return ''.join((url, title, fingerprint))


class CalendarView(BrowserView):
    """ view for aggregating event data into an `.ics` feed """

    def update(self):
        context = self.context
        catalog = getToolByName(context, 'portal_catalog')
        path = '/'.join(context.getPhysicalPath())
        provides = ICalendarSupport.__identifier__
        self.events = catalog(dict(path=path, object_provides=provides))

    def render(self):
        self.update()       # collect events
        context = self.context
        request = self.request
        name = '%s.ics' % context.getId()
        request.RESPONSE.setHeader('Content-Type', 'text/calendar')
        request.RESPONSE.setHeader('Content-Disposition', 'attachment; filename="%s"' % name)
        request.RESPONSE.write(self.feeddata())

    @ram.cache(cachekey)
    def feeddata(self):
        context = self.context
        data = cs.ICS_HEADER % dict(prodid=cs.PRODID)
        data += 'X-WR-CALNAME:%s\n' % context.Title()
        data += 'X-WR-CALDESC:%s\n' % context.Description()
        for brain in self.events:
            data += brain.getObject().getICal()
        data += cs.ICS_FOOTER
        return data

    __call__ = render


class TopicCalendarView(CalendarView):
    """ view (on "topic" content) for aggregating event data into
        an `.ics` feed """

    def update(self):
        context = self.context
        catalog = getToolByName(context, 'portal_catalog')
        if 'object_provides' in catalog.indexes():
            query = {'object_provides': ICalendarSupport.__identifier__}
        else:
            query = {'portal_type': 'Event'}
        self.events = context.queryCatalog(**query)



# -*- coding: utf-8 -*-

import unittest

from cStringIO import StringIO
import os

from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
from Products.Archetypes.interfaces.layer import ILayerContainer
from Products.Archetypes import atapi
from plone.app.blob.content import ATBlob
from Products.ATContentTypes.interfaces import IImageContent
from Products.ATContentTypes.interfaces import IATImage
from Products.ATContentTypes.tests import atcttestcase, atctftestcase
from Products.ATContentTypes.tests.utils import dcEdit, PACKAGE_HOME
from Products.PloneTestCase.PloneTestCase import FunctionalTestCase, setupPloneSite
from Testing import ZopeTestCase  # side effect import. leave it here.
from zope.interface.verify import verifyObject

# BBB Zope 2.12
try:
    from Testing.testbrowser import Browser
except ImportError:
    from Products.Five.testbrowser import Browser

# third party extension
import exif


def loadImage(name, size=0):
    """Load image from testing directory
    """
    path = os.path.join(PACKAGE_HOME, 'input', name)
    fd = open(path, 'rb')
    data = fd.read()
    fd.close()
    return data

TEST_EXIF_ERROR = loadImage('canoneye.jpg')
TEST_GIF = loadImage('test.gif')
TEST_GIF_LEN = len(TEST_GIF)
TEST_DIV_ERROR = loadImage('divisionerror.jpg')
TEST_JPEG_FILE = open(os.path.join(PACKAGE_HOME, 'input', 'canoneye.jpg'), 'rb')
# XXX replace it by an image with exif informations but w/o
# the nasty error ...
TEST_JPEG = loadImage('canoneye.jpg')
TEST_JPEG_LEN = len(TEST_JPEG)


def editATCT(obj):
    obj.setImage(TEST_GIF, content_type="image/gif")
    dcEdit(obj)

tests = []


setupPloneSite()


class TestIDFromTitle(FunctionalTestCase):
    """Browsertests to make sure ATImages derive their default IDs from their titles"""
    # TODO: Merge into TestATImageFunctional, below.

    def afterSetUp(self):
        self.userId = 'fred'
        self.password = 'secret'
        self.portal.acl_users.userFolderAddUser(self.userId, self.password, ['Manager'], [])
        self.browser = Browser()
        self._log_in()

    def _log_in(self):
        """Log in as someone who can add new Images."""
        self.browser.open(self.portal.absolute_url())
        self.browser.getLink('Log in').click()
        self.browser.getControl('Login Name').value = self.userId
        self.browser.getControl('Password').value = self.password
        self.browser.getControl('Log in').click()

    def _make_image(self, title, filename='canoneye.jpg'):
        """Add a new Image at the root of the Plone site."""
        self.browser.open(self.portal.absolute_url() + '/createObject?type_name=Image')
        self.browser.getControl('Title').value = title
        image = self.browser.getControl(name='image_file')
        image.filename = filename
        TEST_JPEG_FILE.seek(0)
        image.value = TEST_JPEG_FILE
        self.browser.getControl('Save').click()

    def test_image_id_from_filename_and_title(self):
        # Get ID from filename:
        self._make_image('')
        self.assertTrue('canoneye.jpg' in self.browser.url)

        # Get ID from title.
        # As a side effect, make sure the ID validator doesn't overzealously
        # deny our upload of something else called canoneye.jpg, even though
        # we're not going to compute its ID from its filename.
        self._make_image('Wonderful Image')
        self.assertTrue('/wonderful-image' in self.browser.url)

    def test_image_id_from_unicode_title(self):
        self._make_image('', filename=u'Pictüre 1.png'.encode('utf-8'))
        normalized = 'Picture%201.png'
        self.assertTrue(normalized in self.browser.url)

tests.append(TestIDFromTitle)


class TestSiteATImage(atcttestcase.ATCTTypeTestCase):

    klass = ATBlob
    portal_type = 'Image'
    title = 'Image'
    meta_type = 'ATImage'

    def test_implementsImageContent(self):
        iface = IImageContent
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_implementsATImage(self):
        iface = IATImage
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_edit(self):
        new = self._ATCT
        editATCT(new)

    def test_getEXIF(self):
        # NOTE: not a real test
        exif_data = self._ATCT.getEXIF()
        self.assertTrue(isinstance(exif_data, dict), type(exif_data))

    def test_exifOrientation(self):
        # NOTE: not a real test
        r, m = self._ATCT.getEXIFOrientation()

    def test_transform(self):
        self._ATCT.update(image=TEST_GIF)
        # NOTE: not a real test
        self._ATCT.transformImage(2)

    def test_autotransform(self):
        # NOTE: not a real test
        self._ATCT.autoTransformImage()

    def test_broken_pil(self):
        # PIL has a nasty bug when the image ratio is too extrem like 300x15:
        # Module PIL.Image, line 1136, in save
        # Module PIL.PngImagePlugin, line 510, in _save
        # Module PIL.ImageFile, line 474, in _save
        # SystemError: tile cannot extend outside image
        atct = self._ATCT

        # test upload
        atct.setImage(TEST_GIF, mimetype='image/gif', filename='test.gif')
        self.assertEqual(atct.getImage().data, TEST_GIF)

    def test_division_by_0_pil(self):
        # pil generates a division by zero error on some images
        atct = self._ATCT

        # test upload
        atct.setImage(TEST_DIV_ERROR, mimetype='image/jpeg',
                      filename='divisionerror.jpg')
        self.assertEqual(atct.getImage().data, TEST_DIV_ERROR)

    def test_get_size(self):
        atct = self._ATCT
        editATCT(atct)
        self.assertEqual(len(TEST_GIF), TEST_GIF_LEN)
        self.assertEqual(atct.get_size(), TEST_GIF_LEN)

    def test_schema_marshall(self):
        atct = self._ATCT
        schema = atct.Schema()
        marshall = schema.getLayerImpl('marshall')
        marshallers = [atapi.PrimaryFieldMarshaller]
        try:
            from Products.Marshall import ControlledMarshaller
            marshallers.append(ControlledMarshaller)
        except ImportError:
            pass
        self.assertTrue(isinstance(marshall, tuple(marshallers)), marshall)

    def test_dcEdit(self):
        new = self._ATCT
        new.setImage(TEST_GIF, content_type="image/gif")
        dcEdit(new)

    def test_broken_exif(self):
        # This test fails even with the 2005.05.12 exif version from
        #    http://home.cfl.rr.com/genecash/
        f = StringIO(TEST_EXIF_ERROR)
        exif_data = exif.process_file(f, debug=False)
        # probably want to add some tests on returned data. Currently gives
        #  ValueError in process_file

    def test_exif_upload(self):
        atct = self._ATCT
        atct._image_exif = None

        # string upload
        atct.setImage(TEST_JPEG)
        self.assertTrue(len(atct.getEXIF()), atct.getEXIF())
        atct._image_exif = None

        # file upload
        atct.setImage(TEST_JPEG_FILE)
        self.assertTrue(len(atct.getEXIF()), atct.getEXIF())
        atct._image_exif = None

        # Pdata upload
        from OFS.Image import Pdata
        pd = Pdata(TEST_JPEG)
        atct.setImage(pd)
        self.assertTrue(len(atct.getEXIF()), atct.getEXIF())
        atct._image_exif = None

        # ofs image upload
        ofs = atct.getImage()
        atct.setImage(ofs)
        self.assertTrue(len(atct.getEXIF()), atct.getEXIF())
        atct._image_exif = None

tests.append(TestSiteATImage)


class TestATImageFields(atcttestcase.ATCTFieldTestCase):

    # Title is not a required field, since we don't require them
    # on File/Image - they are taken from the filename if not present.
    # "Add the comment 'damn stupid fucking test'" -- optilude ;)
    def test_title(self):
        pass

    def afterSetUp(self):
        atcttestcase.ATCTFieldTestCase.afterSetUp(self)
        self._dummy = self.createDummy(klass=ATBlob, subtype='Image')

    def test_imageField(self):
        dummy = self._dummy
        field = dummy.getField('image')

        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.required == 1, 'Value is %s' % field.required)
        self.assertTrue(field.default == None, 'Value is %s' % str(field.default))
        self.assertTrue(field.searchable == 0, 'Value is %s' % field.searchable)
        self.assertTrue(field.vocabulary == (),
                        'Value is %s' % str(field.vocabulary))
        self.assertTrue(field.enforceVocabulary == 0,
                        'Value is %s' % field.enforceVocabulary)
        self.assertTrue(field.multiValued == 0,
                        'Value is %s' % field.multiValued)
        self.assertTrue(field.isMetadata == 0, 'Value is %s' % field.isMetadata)
        self.assertTrue(field.accessor == 'getImage',
                        'Value is %s' % field.accessor)
        self.assertTrue(field.mutator == 'setImage',
                        'Value is %s' % field.mutator)
        self.assertTrue(field.read_permission == View,
                        'Value is %s' % field.read_permission)
        self.assertTrue(field.write_permission == ModifyPortalContent,
                        'Value is %s' % field.write_permission)
        self.assertTrue(field.generateMode == 'veVc',
                        'Value is %s' % field.generateMode)
        self.assertTrue(field.force == '', 'Value is %s' % field.force)
        self.assertTrue(field.type == 'blob', 'Value is %s' % field.type)
        self.assertTrue(isinstance(field.storage, atapi.AnnotationStorage),
                        'Value is %s' % type(field.storage))
        self.assertTrue(field.getLayerImpl('storage') == atapi.AnnotationStorage(migrate=True),
                        'Value is %s' % field.getLayerImpl('storage'))
        self.assertTrue(ILayerContainer.providedBy(field))
        self.assertTrue(field.validators == "(('isNonEmptyFile', V_REQUIRED), ('checkImageMaxSize', V_REQUIRED))",
                        'Value is %s' % str(field.validators))
        self.assertTrue(isinstance(field.widget, atapi.ImageWidget),
                        'Value is %s' % id(field.widget))
        vocab = field.Vocabulary(dummy)
        self.assertTrue(isinstance(vocab, atapi.DisplayList),
                        'Value is %s' % type(vocab))
        self.assertTrue(tuple(vocab) == (), 'Value is %s' % str(tuple(vocab)))
        self.assertTrue(field.primary == 1, 'Value is %s' % field.primary)

tests.append(TestATImageFields)


class TestATImageFunctional(atctftestcase.ATCTIntegrationTestCase):

    portal_type = 'Image'
    views = ('image_view', 'download', 'atct_image_transform')

    def afterSetUp(self):
        atctftestcase.ATCTIntegrationTestCase.afterSetUp(self)
        self.obj.setImage(TEST_GIF, content_type="image/gif")
        dcEdit(self.obj)

    def test_url_returns_image(self):
        response = self.publish(self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    def test_bobo_hook_security(self):
        # Make sure that users with 'View' permission can use the
        # bobo_traversed image scales, even if denied to anonymous
        response1 = self.publish(self.obj_path + '/image', self.basic_auth)
        self.assertEqual(response1.getStatus(), 200)  # OK
        # deny access to anonymous
        self.obj.manage_permission('View', ['Manager', 'Member'], 0)
        response2 = self.publish(self.obj_path + '/image', self.basic_auth)
        # Should be allowed for member
        self.assertEqual(response2.getStatus(), 200)  # OK
        # Should fail for anonymous
        response3 = self.publish(self.obj_path + '/image')
        self.assertEqual(response3.getStatus(), 302)
        bobo_exception = response3.getHeader('bobo-exception-type')
        self.assertTrue('Unauthorized' in bobo_exception)

tests.append(TestATImageFunctional)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



from zope.interface import Interface
from zope.interface import Attribute

from Products.Archetypes.interfaces import IBaseContent
from Products.Archetypes.interfaces import IBaseFolder
from Products.CMFDynamicViewFTI.interfaces import ISelectableBrowserDefault
from Products.Archetypes.interfaces import IATHistoryAware
from Products.CMFPlone.interfaces import ISelectableConstrainTypes


class IATContentType(ISelectableBrowserDefault, IBaseContent):
    """Marker interface for AT Content Types
    """

    default_view = Attribute('''Default view template - used for dynamic view''')
    suppl_views = Attribute('''Supplementary views - used for dynamic view''')

    _atct_newTypeFor = Attribute('''XXX''')

    assocMimetypes = Attribute('''A tuple of mimetypes that are associated
                                  with this type. Format: ('bar/foo', 'foo/*',)
                               ''')

    assocFileExt = Attribute('''A tuple of file extensions that are associated
                                with this type. Format: ('jpeg', 'png',)
                             ''')

    cmf_edit_kws = Attribute('''List of keyword names.

    If one of this kw names is used with edit() then the cmf_edit method is
    called.
    ''')


class IHistoryAware(IATHistoryAware):
    """History awareness marker interface
    """

    def getHistorySource():
        """get source for HistoryAwareMixin

        Must return a (raw) string
        """

    def getLastEditor():
        """Returns the user name of the last editor.

        Returns None if no last editor is known.
        """

    def getDocumentComparisons(max=10, filterComment=0):
        """Get history as unified diff
        """


class ICalendarSupport(Interface):
    """Calendar import/export
    """


class ITextContent(Interface):
    """Interface for types containing text
    """

    def getText(**kwargs):
        """
        """

    def setText(value, **kwargs):
        """
        """

    def CookedBody(stx_level='ignored'):
        """
        """

    def EditableBody():
        """
        """


class IATCTTool(Interface):
    """
    """



# -*- coding: UTF-8 -*-

from Products.ATContentTypes.tests.atcttestcase import ATCTFunctionalSiteTestCase

FILES = [
    'topictool.txt', 'portaltype_criterion.txt', 'webdav.txt', 'http_access.txt',
    'reindex_sanity.txt', 'uploading.txt', 'browser_collection_views.txt',
    # traversal.txt registers the browser page "document_view", and this registration
    # stays active in different doctests, so we make sure to include it last.
    'traversal.txt',
]

import doctest
OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE |
               doctest.REPORT_NDIFF)


def test_suite():
    import unittest
    suite = unittest.TestSuite()
    from Testing.ZopeTestCase import FunctionalDocFileSuite as FileSuite
    for testfile in FILES:
        suite.addTest(FileSuite(testfile,
                                optionflags=OPTIONFLAGS,
                                package="Products.ATContentTypes.tests",
                                test_class=ATCTFunctionalSiteTestCase)
                     )
    return suite



from zope.interface import implements

from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent

from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import BooleanField, StringField
from Products.Archetypes.atapi import BooleanWidget, StringWidget

from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import PATH_INDICES
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics


from Products.CMFCore.utils import getToolByName

ATRelativePathCriterionSchema = ATBaseCriterionSchema + Schema((
    StringField('relativePath',
                default='..',
                widget=StringWidget(label='Relative path',
                                    label_msgid="label_relativepath_criteria_customrelativepath",
                                    description_msgid="help_relativepath_criteria_customrelativepath",
                                    i18n_domain="plone",
                                    description="Enter a relative path e.g.: <br /> '..' for the parent folder <br /> '../..' for the parent's parent <br />'../somefolder' for a sibling folder")),
    BooleanField('recurse',
                mode="rw",
                write_permission=ChangeTopics,
                accessor="Recurse",
                default=False,
                widget=BooleanWidget(
                    label="Search Sub-Folders",
                    label_msgid="label_path_criteria_recurse",
                    description="",
                    description_msgid="help_path_criteria_recurse",
                    i18n_domain="plone"),
                ),
    ))


class ATRelativePathCriterion(ATBaseCriterion):
    """A path criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATRelativePathCriterionSchema
    meta_type = 'ATRelativePathCriterion'
    archetype_name = 'Relative Path Criterion'
    shortDesc = 'Location in site relative to the current location'

    def getNavTypes(self):
        ptool = self.plone_utils
        nav_types = ptool.typesToList()
        return nav_types

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems(self):
        result = []
        depth = (not self.Recurse() and 1) or -1
        relPath = self.getRelativePath()

        # sanitize a bit: you never know, with all those windoze users out there
        relPath = relPath.replace("\\", "/")

        # get the path to the portal object
        portalPath = list(getToolByName(self, 'portal_url').getPortalObject().getPhysicalPath())

        if relPath[0] == '/':
            # someone didn't enter a relative path.
            # simply use that one, relative to the portal
            path = '/'.join(portalPath) + relPath
        else:
            folders = relPath.split('/')

            # set the path to the collections path
            path = list(aq_parent(self).getPhysicalPath())

            # now construct an aboslute path based on the relative custom path
            # eat away from 'path' whenever we encounter a '..' in the relative path
            # apend all other elements other than ..
            for folder in folders:
                if folder == '..':
                    # chop off one level from path
                    if path == portalPath:
                        # can't chop off more
                        # just return this path and leave the loop
                        break
                    else:
                        path = path[:-1]
                elif folder == '.':
                    # don't really need this but for being complete
                    # strictly speaking some user may use a . aswell
                    pass  # do nothing
                else:
                    path.append(folder)
            path = '/'.join(path)

        if path is not '':
            result.append((self.Field(), {'query': path, 'depth': depth}))

        return tuple(result)


registerCriterion(ATRelativePathCriterion, PATH_INDICES)



from zope.interface import implements
from zope.interface import Interface

from Products.GenericSetup.interfaces import IFilesystemExporter

# BBB: Leaving this old 'temporary hack' in, in case people are relying on it.
# It's no longer used in core.


class IDisabledExport(Interface):
    pass


class NullExporterAdapter(object):
    """Dummy exporter that does nothing
    """

    implements(IFilesystemExporter)

    def __init__(self, context):
        self.context = context

    def export(self, export_context, subdir):
        pass

    def listExportableItems(self):
        return []



from zope.interface import Interface


class IImageContent(Interface):
    """Interface for types containing an image
    """

    def getImage(**kwargs):
        """
        """

    def setImage(value, **kwargs):
        """
        """

    def tag(**kwargs):
        """
        """


class IATImage(IImageContent):
    """AT Image marker Interface
    """



import difflib

from zope.interface import implements

from DocumentTemplate.DT_Util import html_quote
from App.class_init import InitializeClass

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import View
from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import ATHistoryAwareMixin
from Products.ATContentTypes import permission as ATCTPermissions
from Products.ATContentTypes.interfaces import IHistoryAware


class HistoryAwareMixin(ATHistoryAwareMixin):
    """History aware mixin class

    Shows a unified diff history of the content

    This mixin is using some low level functions of the ZODB to get the last
    transaction states (versions) of the current object. Older histories will
    disapear after packing the database so DO NOT rely on the history
    functionality. It's more a gimmick and nice helper to reviewers and site
    managers.
    """

    implements(IHistoryAware)

    security = ClassSecurityInfo()

    actions = ({
        'id': 'history',
        'name': 'History',
        'action': 'string:${object_url}/atct_history',
        'permissions': (ATCTPermissions.ViewHistory, ),
        'visible': False,
         },
    )

    security.declarePrivate('getHistorySource')
    def getHistorySource(self):
        """get source for HistoryAwareMixin

        Must return a (raw) string
        """
        primary = self.getPrimaryField()
        if primary:
            return primary.getRaw(self)
        else:
            return ''

    security.declareProtected(View, 'getLastEditor')
    def getLastEditor(self):
        """Returns the user name of the last editor.

        Returns None if no last editor is known.
        """
        histories = list(self.getHistories(1))
        if not histories:
            return None
        user = histories[0][3].split(" ")[-1].strip()
        return  user

    security.declareProtected(ATCTPermissions.ViewHistory, 'getDocumentComparisons')
    def getDocumentComparisons(self, max=10, filterComment=0):
        """Get history as unified diff
        """
        mTool = getToolByName(self, 'portal_membership')

        histories = list(self.getHistories())
        if max > len(histories):
            max = len(histories)

        lst = []

        for revisivon in range(1, max):

            oldObj, oldTime, oldDesc, oldUser = histories[revisivon]
            newObj, newTime, newDesc, newUser = histories[revisivon - 1]

            oldText = oldObj.getHistorySource().split("\n")
            newText = newObj.getHistorySource().split("\n")
            # newUser is a string 'user' or 'folders to acl_users user'
            member = mTool.getMemberById(newUser.split(' ')[-1])

            lines = [
                     html_quote(line)
                     for line in difflib.unified_diff(oldText, newText)
                    ][3:]

            description = newDesc
            if filterComment:
                relativUrl = self.absolute_url(1)
                description = '<br />\n'.join(
                              [line
                               for line in description.split('\n')
                               if line.find(relativUrl) != -1]
                              )
            else:
                description.replace('\n', '<br />\n')

            if lines:
                lst.append({
                            'lines': lines,
                            'oldTime': oldTime,
                            'newTime': newTime,
                            'description': description,
                            'user': newUser,
                            'member': member
                           })
        return lst

InitializeClass(HistoryAwareMixin)



## Script (Python) "atctListAlbum"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=images=0, folders=0, subimages=0, others=0
##title=Helper method for photo album view
##
result = {}
is_topic = context.portal_type == 'Topic'

if images:
    if is_topic:
        result['images'] = context.queryCatalog(portal_type=('Image',),
                                                full_objects=True)
    else:
        result['images'] = context.getFolderContents({'portal_type': ('Image',)},
                                                     full_objects=True)

if folders:
    # We don't need the full objects for the folders
    if is_topic:
        result['folders'] = context.queryCatalog(portal_type='Folder')
    else:
        result['folders'] = context.getFolderContents({'portal_type': ('Folder',)})

if subimages:
    # Handle brains or objects
    if getattr(context, 'getPath', None) is not None:
        path = context.getPath()
    else:
        path = '/'.join(context.getPhysicalPath())
    # Explicitly set path to remove default depth
    if is_topic:
        result['subimages'] = context.queryCatalog(portal_type=('Image',),
                                                   path=path)
    else:
        result['subimages'] = context.getFolderContents({'portal_type': ('Image',),
                                                 'path': path})

if others:
    searchContentTypes = context.plone_utils.getUserFriendlyTypes()
    filtered = [p_type for p_type in searchContentTypes
                if p_type not in ('Image', 'Folder',)]
    if filtered:
        # We don't need the full objects for the folder_listing
        if is_topic:
            result['others'] = context.queryCatalog(portal_type=filtered)
        else:
            result['others'] = context.getFolderContents({'portal_type': filtered})
    else:
        result['others'] = ()

return result



from zope.interface import implements

from App.class_init import InitializeClass
from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.criteria import _criterionRegistry
from AccessControl import ClassSecurityInfo
from Persistence import Persistent
from OFS.SimpleItem import SimpleItem
from ExtensionClass import Base

from Products.ATContentTypes.config import TOOLNAME
from Products.ATContentTypes.interfaces import IATCTTopicsTool
from Products.Archetypes.atapi import DisplayList
from Products.CMFCore.permissions import ManagePortal


class TopicIndex(SimpleItem, Persistent):

    def __init__(self, index, friendlyName='', description='', enabled=False, criteria=()):
        self.index = index
        self.friendlyName = friendlyName
        self.description = description
        self.enabled = enabled
        self.criteria = tuple(criteria)


class ATTopicsTool(Base):
    """This tool makes it possible to manage the indexes that are used
       inside topics and allows you to enter a friendly name instead of
       cryptic indexes.
    """
    implements(IATCTTopicsTool)

    security = ClassSecurityInfo()

    # is used in ATPortalTypeCriterion to control which types are allowed to search with
    # until this is fixed in CMF or whatever, this is the way to go.
    allowed_portal_types = []

    id = TOOLNAME
    meta_type = 'ATTopics Tool'
    title = 'ATTopics Tool'
    plone_tool = True

    def __init__(self):
        self.topic_indexes = {}
        self.topic_metadata = {}
        self.allowed_portal_types = []

    def getCriteriaForIndex(self, index, as_dict=False):
        """ Returns the valid criteria for a given index """
        catalog_tool = getToolByName(self, 'portal_catalog')
        try:
            indexObj = catalog_tool.Indexes[index]
        except KeyError:
            return ()
        criteria = tuple(_criterionRegistry.criteriaByIndex(indexObj.meta_type))
        search_criteria = _criterionRegistry.listSearchTypes()
        if as_dict:
            criteria = [{'name': a, 'description': _criterionRegistry[a].shortDesc}
                                for a in criteria if a in search_criteria]
        else:
            criteria = [a for a in criteria if a in search_criteria]
        criteria.sort()
        return criteria

    security.declareProtected(ManagePortal, 'addIndex')
    def addIndex(self, index, friendlyName='', description='', enabled=False, criteria=None):
        """ Add a new index along with descriptive information to the index
            registry """
        if criteria is None:
            criteria = self.getCriteriaForIndex(index)
        if index in self.topic_indexes:
            objIndex = self.topic_indexes[index]
            objIndex.friendlyName = friendlyName
            objIndex.description = description
            objIndex.enabled = enabled
            objIndex.criteria = tuple(criteria)
        else:
            objIndex = TopicIndex(index, friendlyName, description, enabled, criteria)

        self.topic_indexes[index] = objIndex
        self._p_changed = True

    security.declareProtected(ManagePortal, 'addMetadata')
    def addMetadata(self, metadata, friendlyName='', description='', enabled=False):
        """ Add a new metadata field along with descriptive information to the
            metadata registry """
        if metadata in self.topic_metadata:
            objMeta = self.topic_metadata[metadata]
            objMeta.friendlyName = friendlyName
            objMeta.description = description
            objMeta.enabled = enabled
        else:
            objMeta = TopicIndex(metadata, friendlyName, description, enabled)

        self.topic_metadata[metadata] = objMeta
        self._p_changed = True

    security.declareProtected(ManagePortal, 'updateIndex')
    def updateIndex(self, index, friendlyName=None, description=None, enabled=None, criteria=None):
        """ Updates an existing index in the registry, unrecognized values are
            added """
        indexes = self.topic_indexes
        if friendlyName == None:
            friendlyName = indexes[index].friendlyName
        if description == None:
            description = indexes[index].description
        if enabled == None:
            enabled = indexes[index].enabled
        if criteria == None:
            criteria = indexes[index].criteria

        self.addIndex(index, friendlyName, description, enabled, criteria)

    security.declareProtected(ManagePortal, 'updateMetadata')
    def updateMetadata(self, metadata, friendlyName=None, description=None, enabled=None):
        """ Updates an existing metadata field in the registry, unrecognized values are
            added """
        meta = self.topic_metadata
        if friendlyName == None:
            friendlyName = meta[metadata].friendlyName
        if description == None:
            description = meta[metadata].description
        if enabled == None:
            enabled = meta[metadata].enabled
        self.addMetadata(metadata, friendlyName, description, enabled)

    security.declareProtected(ManagePortal, 'removeIndex')
    def removeIndex(self, index):
        """ Removes an existing index from the registry """
        if index in self.topic_indexes:
            del self.topic_indexes[index]
            self._p_changed = True

    security.declareProtected(ManagePortal, 'removeMetadata')
    def removeMetadata(self, metadata):
        """ Removes an existing metadata field from the registry """
        if metadata in self.topic_metadata:
            del self.topic_metadata[metadata]
            self._p_changed = True

    security.declareProtected(ManagePortal, 'createInitialIndexes')
    def createInitialIndexes(self):
        """ create indexes for all indexes in the catalog """
        indexes = self.listCatalogFields()
        for i in indexes:
            if i not in self.topic_indexes:
                enabled = False
                self.addIndex(i, friendlyName='', enabled=enabled)
        return True

    security.declareProtected(ManagePortal, 'createInitialMetadata')
    def createInitialMetadata(self):
        """ create metadata for all indexes in the catalog """
        metas = self.listCatalogMetadata()
        for i in metas:
            if i not in self.topic_metadata:
                enabled = False
                self.addMetadata(i, friendlyName='', enabled=enabled)
        return True

    security.declarePrivate('listCatalogFields')
    def listCatalogFields(self):
        """ Return a list of fields from portal_catalog. """
        pcatalog = getToolByName(self, 'portal_catalog')
        available = pcatalog.indexes()
        val = [field for field in available]
        val.sort()
        return val

    security.declarePrivate('listCatalogMetadata')
    def listCatalogMetadata(self):
        """ Return a list of columns from portal_catalog. """
        pcatalog = getToolByName(self, 'portal_catalog')
        available = pcatalog.schema()
        val = [field for field in available]
        val.sort()
        return val

    def getAllPortalTypes(self):
        """ returns a list of (id, title)-tuples for each type """
        types_tool = getToolByName(self, 'portal_types')
        types = types_tool.listTypeInfo()

        all_types = [(t.id, t.title or t.id) for t in types]
        return all_types

    def getAllowedPortalTypes(self, populate_for_end_usage=1):
        """ Return all portal_types as an (id,title) tuple that are allowed
            to search with """
        all_types = self.getAllPortalTypes()

        if populate_for_end_usage == 0:
            # return whatever is in self.allowed_portal_types and make it a (id, title) tuple
            return [t for t in all_types if t[0] in self.allowed_portal_types]

        if self.allowed_portal_types == []:
            filtered_types = all_types
        else:
            filtered_types = [type for type in all_types if type[0] in self.allowed_portal_types]

        return filtered_types

    def getEnabledIndexes(self):
        """ Returns all TopicIndex objects for enabled indexes """
        indexes = self.topic_indexes
        results = [i for i in indexes.values() if i.enabled]
        return results

    def getEnabledMetadata(self):
        """ Returns all TopicIndex objects for enabled metadata """
        meta = self.topic_metadata
        results = [i for i in meta.values() if i.enabled]
        return results

    def getIndexDisplay(self, enabled=True):
        """ Return DisplayList of Indexes and their friendly names """
        if enabled:
            index_names = self.getIndexes(True)
        else:
            index_names = self.getIndexes()
        index_dict = self.topic_indexes
        indexes = [index_dict[i] for i in index_names]
        field_list = [(f.index, f.friendlyName or f.index) for f in indexes]
        return DisplayList(field_list)

    def getMetadataDisplay(self, enabled=True):
        """ Return DisplayList of Metadata and their friendly names """
        if enabled:
            meta_names = self.getAllMetadata(True)
        else:
            meta_names = self.getAllMetadata()
        meta_dict = self.topic_metadata
        meta = [meta_dict[i] for i in meta_names]
        field_list = [(f.index, f.friendlyName or f.index) for f in meta]
        return DisplayList(field_list)

    def getEnabledFields(self):
        """ Returns a list of tuples containing the index name, friendly name,
            and description for each enabled index. """
        enabledIndexes = self.getEnabledIndexes()
        dec_fields = [(i.friendlyName.lower() or \
                       i.index.lower(), i.index, i.friendlyName or \
                       i.index, i.description) for i in enabledIndexes]
        dec_fields.sort()
        fields = [(a[1], a[2], a[3]) for a in dec_fields]
        return fields

    def getFriendlyName(self, index):
        """ Returns the friendly name for a given index name, or the given
            index if the firendlyname is empty or the index is not recognized
        """
        if index in self.topic_indexes:
            return self.getIndex(index).friendlyName or index
        else:
            return index

    security.declareProtected(ManagePortal, 'getIndexes')
    def getIndexes(self, enabledOnly=False):
        """ Returns the full list of available indexes, optionally filtering
            out those that are not marked enabled """
        if enabledOnly:
            indexes_dec = [(i.index.lower(), i.index) for i in self.getEnabledIndexes()]
        else:
            self.createInitialIndexes()  # update in case of new catalogue indexes
            indexes_dec = [(i.lower(), i) for i in self.topic_indexes.keys()]

        indexes_dec.sort()
        indexes = [i[1] for i in indexes_dec]
        return indexes

    security.declareProtected(ManagePortal, 'getAllMetadata')
    def getAllMetadata(self, enabledOnly=False):
        """ Returns the full list of available metadata fields, optionally
            filtering out those that are not marked enabled """
        if enabledOnly:
            meta_dec = [(i.index.lower(), i.index) for i in self.getEnabledMetadata()]
        else:
            self.createInitialMetadata()  # update in case of new catalogue metadata
            meta_dec = [(i.lower(), i) for i in self.topic_metadata.keys()]

        meta_dec.sort()
        metadata = [i[1] for i in meta_dec]
        return metadata

    def getIndex(self, index):
        """ Returns the TopicIndex object for a given index name """
        if index in self.topic_indexes:
            return self.topic_indexes[index]
        else:
            raise AttributeError('Index ' + str(index) + ' not found')

    def getMetadata(self, metadata):
        """ Returns the TopicIndex object for a given metadata name """
        if metadata in self.topic_metadata:
            return self.topic_metadata[metadata]
        else:
            raise AttributeError('Metadata ' + str(metadata) + ' not found')

    security.declareProtected(ManagePortal, 'manage_saveTopicSetup')
    def manage_saveTopicSetup(self, REQUEST=None):
        """ Set indexes and metadata from form """
        if REQUEST == None:
            return  'Nothing saved.'

        data = REQUEST.get('index', [])
        for index in data:
            enabled = 'enabled' in index
            criteria = index.get('criteria', ())
            self.updateIndex(index['index'], index['friendlyName'], index['description'], enabled, criteria)

        meta = REQUEST.get('metadata', [])
        for metadata in meta:
            enabled = 'enabled' in metadata
            self.updateMetadata(metadata['index'], metadata['friendlyName'], metadata['description'], enabled)
        return True

    security.declareProtected(ManagePortal, 'manage_saveTopicSetupTypes')
    def manage_saveTopicSetupTypes(self, REQUEST=None):
        """ Set portal types from form """
        if REQUEST == None:
            return  'Nothing saved.'

        self.allowed_portal_types = REQUEST.get('allowed_types', [])
        return True

InitializeClass(ATTopicsTool)



import unittest

from Testing import ZopeTestCase  # side effect import. leave it here.
from Products.ATContentTypes.tests import atcttestcase, atctftestcase

from Products.ATContentTypes.tests.utils import dcEdit
from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.folder import ATBTreeFolder
from OFS.interfaces import IOrderedContainer as IOrderedContainer
from Products.ATContentTypes.interfaces import IATFolder
from Products.ATContentTypes.interfaces import IATBTreeFolder

from zope.interface.verify import verifyObject
from Products.ATContentTypes.interfaces import ISelectableConstrainTypes


def editATCT(obj):
    dcEdit(obj)

tests = []


class FolderTestMixin:
    """Contains some general tests for both ATFolder and ATBTreeFolder
    """
    def test_implementsConstrainTypes(self):
        self.assertTrue(ISelectableConstrainTypes.providedBy(self._ATCT))
        self.assertTrue(verifyObject(ISelectableConstrainTypes, self._ATCT))


class TestSiteATFolder(atcttestcase.ATCTTypeTestCase, FolderTestMixin):

    klass = ATFolder
    portal_type = 'Folder'
    title = 'Folder'
    meta_type = 'ATFolder'

    def test_implementsOrderInterface(self):
        self.assertTrue(IOrderedContainer.providedBy(self._ATCT))
        self.assertTrue(verifyObject(IOrderedContainer, self._ATCT))

    def test_implementsATFolder(self):
        iface = IATFolder
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_implementsConstrainTypes(self):
        iface = ISelectableConstrainTypes
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_edit(self):
        new = self._ATCT
        editATCT(new)

    def test_get_size(self):
        atct = self._ATCT
        self.assertEqual(atct.get_size(), 1)

    def test_schema_marshall(self):
        pass

tests.append(TestSiteATFolder)


class TestSiteATBTreeFolder(atcttestcase.ATCTTypeTestCase, FolderTestMixin):

    klass = ATBTreeFolder
    portal_type = 'Large Plone Folder'
    title = 'Large Folder'
    meta_type = 'ATBTreeFolder'

    def test_implementsATBTreeFolder(self):
        iface = IATBTreeFolder
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_implementsConstrainTypes(self):
        iface = ISelectableConstrainTypes
        self.assertTrue(iface.providedBy(self._ATCT))
        self.assertTrue(verifyObject(iface, self._ATCT))

    def test_isOrdered(self):
        self.assertTrue(IOrderedContainer.providedBy(self._ATCT))

    def test_edit(self):
        new = self._ATCT
        editATCT(new)
        self.assertTrue('Test title' == new.Title())
        self.assertTrue('Test description' == new.Description())

    def test_get_size(self):
        atct = self._ATCT
        self.assertEqual(atct.get_size(), 1)

    def test_schema_marshall(self):
        pass

    def test_dictBehavior(self):
        # this test currently fails intentionally (see
        # http://dev.plone.org/collective/changeset/53298).
        # debugging it for a while shows that BTreeFolders don't
        # work as iterators for some strange reason.  while
        # "list(folder.__iter__())" works just fine, "list(folder)"
        # doesn't.  python doesn't seem to recognize the `__iter__`
        # method and instead falls back to its standard iterator
        # cycling the list via `__len__` and `__getitem__`.  however,
        # the interesting bit here is, that "list(folder.aq_base)"
        # does work, so apparently it's acquisition biting us here...
        #
        # meanwhile this bug has been fixed upstream in Zope (see
        # http://svn.zope.org/?rev=94907&view=rev), so the next release
        # should make this test pass
        #
        # update: using zope 2.10.8 makes this test pass indeed...
        self.setRoles(('Manager',))
        self.portal.invokeFactory('Folder', 'f1')
        f1 = self.portal['f1']

        from Products.ATContentTypes.content.document import ATDocument
        new_doc = ATDocument('d1')
        f1['d1'] = new_doc
        new_doc = f1['d1']  # aq-wrap

        self.assertEqual(['d1'], list(f1.keys()))  # keys
        self.assertEqual(['d1'], list(f1.iterkeys()))  # iterkeys
        try:
            self.assertEqual(['d1'], list(f1))  # iter
        except (KeyError, AttributeError):
            print '\nKnown failure: please see comments in `test_dictBehavior`!'
        self.assertEqual(['d1'], list(f1.aq_base))  # iter (this works, weird!)
        self.assertTrue(f1.values()[0].aq_base is new_doc.aq_base)  # values
        self.assertTrue(f1.get('d1').aq_base is new_doc.aq_base)  # get
        self.assertTrue('d1' in f1)  # contains


tests.append(TestSiteATBTreeFolder)


class TestATFolderFields(atcttestcase.ATCTFieldTestCase):

    def afterSetUp(self):
        atcttestcase.ATCTFieldTestCase.afterSetUp(self)
        self._dummy = self.createDummy(klass=ATFolder)

    def test_field_enableConstrainMixin(self):
        pass
        #self.fail('not implemented')

    def test_field_locallyAllowedTypes(self):
        pass
        #self.fail('not implemented')

tests.append(TestATFolderFields)


class TestATBTreeFolderFields(TestATFolderFields):

    def afterSetUp(self):
        atcttestcase.ATCTFieldTestCase.afterSetUp(self)
        self._dummy = self.createDummy(klass=ATBTreeFolder)

tests.append(TestATBTreeFolderFields)


class TestATFolderFunctional(atctftestcase.ATCTIntegrationTestCase):

    portal_type = 'Folder'
    views = ('folder_listing', 'atct_album_view', )

    def test_dynamic_view_without_view(self):
        # dynamic view mixin should work
        response = self.publish('%s/' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)

    def test_selectViewTemplate(self):
        # create an object using the createObject script
        self.publish(self.obj_path +
                     '/selectViewTemplate?templateId=atct_album_view',
                     self.owner_auth)
        self.assertEqual(self.obj.getLayout(), 'atct_album_view')

tests.append(TestATFolderFunctional)


class TestATBTreeFolderFunctional(atctftestcase.ATCTIntegrationTestCase):

    portal_type = 'Large Plone Folder'
    views = ('folder_listing', 'atct_album_view', )

    def test_templatemixin_view_without_view(self):
        # template mixin magic should work
        response = self.publish('%s/' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)

tests.append(TestATBTreeFolderFunctional)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



# -*- coding: utf-8 -*-

from unittest import defaultTestLoader
from zope.interface import classImplements
from zope.component import getMultiAdapter
from zope.publisher.browser import TestRequest
from zope.annotation.interfaces import IAttributeAnnotatable

from Products.ATContentTypes.tests.atcttestcase import ATCTSiteTestCase
from Products.ATContentTypes.interfaces import ICalendarSupport
from Products.ATContentTypes.browser.calendar import cachekey


def makeResponse(request):
    """ create a fake request and set up logging of output """
    headers = {}
    output = []

    class Response:
        def setHeader(self, header, value):
            headers[header] = value

        def write(self, msg):
            output.append(msg)

    request.RESPONSE = Response()
    return headers, output, request


class EventCalendarTests(ATCTSiteTestCase):

    def afterSetUp(self):
        super(EventCalendarTests, self).afterSetUp()
        folder = self.folder
        event1 = folder[folder.invokeFactory('Event',
            id='ploneconf2007', title='Plone Conf 2007',
            startDate='2007/10/10', endDate='2007/10/12', location='Naples',
            eventUrl='http://plone.org/events/conferences/2007-naples')]
        event2 = folder[folder.invokeFactory('Event',
            id='ploneconf2008', title='Plone Conf 2008',
            startDate='2008/10/08', endDate='2008/10/10', location='DC',
            eventUrl='http://plone.org/events/conferences/2008-washington-dc')]
        classImplements(TestRequest, IAttributeAnnotatable)

    def testCalendarView(self):
        view = getMultiAdapter((self.folder, TestRequest()), name='ics_view')
        view.update()
        self.assertEqual(len(view.events), 2)
        self.assertEqual(sorted([e.Title for e in view.events]),
            ['Plone Conf 2007', 'Plone Conf 2008'])

    def testCalendarViewForTopic(self):
        self.setRoles(('Manager',))
        folder = self.folder
        topic = self.folder[self.folder.invokeFactory('Topic', id='dc')]
        crit = topic.addCriterion('SearchableText', 'ATSimpleStringCriterion')
        crit.setValue('DC')
        view = getMultiAdapter((topic, TestRequest()), name='ics_view')
        view.update()
        self.assertEqual(len(view.events), 1)
        self.assertEqual(sorted([e.Title for e in view.events]),
            ['Plone Conf 2008'])
        folder[folder.invokeFactory('Event',
            id='inaug09', title='Inauguration Day 2009',
            startDate='2009/01/20', endDate='2009/01/20', location='DC')]
        view.update()
        self.assertEqual(len(view.events), 2)
        self.assertEqual(sorted([e.Title for e in view.events]),
            ['Inauguration Day 2009', 'Plone Conf 2008'])

    def testDuplicateQueryParameters(self):
        self.setRoles(('Manager',))
        folder = self.folder
        topic = self.folder[self.folder.invokeFactory('Topic', id='dc')]
        crit = topic.addCriterion('portal_type', 'ATSimpleStringCriterion')
        crit.setValue('Event')
        crit = topic.addCriterion('object_provides', 'ATSimpleStringCriterion')
        crit.setValue(ICalendarSupport.__identifier__)
        query = topic.buildQuery()
        self.assertEqual(len(query), 2)
        self.assertEqual(query['portal_type'], 'Event')
        self.assertEqual(query['object_provides'], ICalendarSupport.__identifier__)
        view = getMultiAdapter((topic, TestRequest()), name='ics_view')
        view.update()
        self.assertEqual(len(view.events), 2)
        self.assertEqual(sorted([e.Title for e in view.events]),
            ['Plone Conf 2007', 'Plone Conf 2008'])

    def checkOrder(self, text, *order):
        for item in order:
            position = text.find(item)
            self.assertTrue(position >= 0,
                'menu item "%s" missing or out of order' % item)
            text = text[position:]

    def testRendering(self):
        headers, output, request = makeResponse(TestRequest())
        view = getMultiAdapter((self.folder, request), name='ics_view')
        view.render()
        self.assertEqual(len(headers), 2)
        self.assertEqual(headers['Content-Type'], 'text/calendar')
        self.checkOrder(''.join(output),
            'BEGIN:VCALENDAR',
            'BEGIN:VEVENT',
            'SUMMARY:Plone Conf 2007',
            'LOCATION:Naples',
            'URL:http://plone.org/events/conferences/2007-naples',
            'END:VEVENT',
            'BEGIN:VEVENT',
            'SUMMARY:Plone Conf 2008',
            'LOCATION:DC',
            'END:VEVENT',
            'END:VCALENDAR')

    def testCalendarInfo(self):
        self.folder.processForm(values={'title': 'Foo', 'description': 'Bar'})
        headers, output, request = makeResponse(TestRequest())
        view = getMultiAdapter((self.folder, request), name='ics_view')
        view.render()
        self.checkOrder(''.join(output),
            'BEGIN:VCALENDAR',
            'X-WR-CALNAME:Foo',
            'X-WR-CALDESC:Bar',
            'BEGIN:VEVENT',
            'BEGIN:VEVENT',
            'END:VCALENDAR')
        # another folder should have another name, even though the set
        # of events might be the same...
        headers, output, request = makeResponse(TestRequest())
        view = getMultiAdapter((self.portal, request), name='ics_view')
        view.render()
        self.checkOrder(''.join(output),
            'BEGIN:VCALENDAR',
            'X-WR-CALNAME:Plone site',
            'X-WR-CALDESC:',
            'BEGIN:VEVENT',
            'BEGIN:VEVENT',
            'END:VCALENDAR')
        # changing the title should be immediately reflected...
        self.folder.processForm(values={'title': 'Föö!!'})
        headers, output, request = makeResponse(TestRequest())
        view = getMultiAdapter((self.folder, request), name='ics_view')
        view.render()
        self.checkOrder(''.join(output),
            'BEGIN:VCALENDAR',
            'X-WR-CALNAME:Föö!!',
            'X-WR-CALDESC:Bar',
            'BEGIN:VEVENT',
            'BEGIN:VEVENT',
            'END:VCALENDAR')

    def testRenderingForTopic(self):
        self.setRoles(('Manager',))
        folder = self.folder
        topic = self.folder[self.folder.invokeFactory('Topic', id='dc')]
        crit = topic.addCriterion('SearchableText', 'ATSimpleStringCriterion')
        crit.setValue('DC')
        headers, output, request = makeResponse(TestRequest())
        view = getMultiAdapter((topic, request), name='ics_view')
        view.render()
        self.assertEqual(len(headers), 2)
        self.assertEqual(headers['Content-Type'], 'text/calendar')
        self.checkOrder(''.join(output),
            'BEGIN:VCALENDAR',
            'BEGIN:VEVENT',
            'SUMMARY:Plone Conf 2008',
            'LOCATION:DC',
            'URL:http://plone.org/events/conferences/2008-washington-dc',
            'END:VEVENT',
            'END:VCALENDAR')
        lines = ''.join(output).splitlines()
        self.assertEqual(len([l for l in lines if l == 'BEGIN:VEVENT']), 1)

    def testCacheKey(self):
        headers, output, request = makeResponse(TestRequest())
        view = getMultiAdapter((self.folder, request), name='ics_view')
        # calculate original key for caching...
        view.update()
        key1 = cachekey(None, view)
        # a second invocation should return the same key...
        view.update()
        key2 = cachekey(None, view)
        self.assertEqual(key1, key2)
        # even with a new view...
        headers, output, request = makeResponse(TestRequest())
        view = getMultiAdapter((self.folder, request), name='ics_view')
        view.update()
        key3 = cachekey(None, view)
        self.assertEqual(key1, key3)
        # however, if one of the object gets changed, the key should as well
        self.folder.ploneconf2007.processForm(values={'location': 'Naples, Italy'})
        view.update()
        key4 = cachekey(None, view)
        self.assertNotEqual(key1, key4)
        # the same goes if another one is added
        self.folder[self.folder.invokeFactory('Event',
            id='ploneconf2009', title='Plone Conf 2009',
            startDate='2008/10/28', endDate='2008/10/30', location='Budapest',
            eventUrl='http://plone.org/events/conferences/2009')]
        view.update()
        key5 = cachekey(None, view)
        self.assertNotEqual(key1, key5)
        self.assertNotEqual(key4, key5)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
